{ Trace/Assert tools. To be completed!
  don't provide a window handle on formdestroy methods
  when calling Trace cuz they aren't valid any more.
}

unit DbgTrace;

interface

uses
  {$IFDEF WIN32} Windows,
  {$ELSE}        WinTypes, WinProcs,
  {$ENDIF}
  SysUtils, Forms;

type

  { Exceptions }
  EDBGTRACE    = class(Exception);
    EDUP       = class(EDBGTRACE);
      EDupCode = class(EDUP);
      EDupStr  = class(EDUP);
    EASSERT    = class(EDBGTRACE);

type

  TAssertProc  = procedure (Assertion: Boolean; RetAddress: Pointer; AErrorCode: Word);

{$IFDEF DBGTRACE}
procedure RegisterErrorCode (ErrorCode: Word);
function  ReturnAddress: Pointer;
procedure Trace(WndOwner: HWND; RetAddress: Pointer; ErrorCode: Integer);
procedure TraceFmt(WndOwner: HWND; RetAddress: Pointer; ErrorCode: Integer;
                   const Args: array of const);
procedure Assert(Assertion: Boolean; RetAddress: Pointer; AErrorCode: Word);

const
  Require: TAssertProc = Assert;
  Ensure: TAssertProc = Assert;
{$ENDIF}

implementation

{$IFDEF DBGTRACE}
  {$IFDEF PROFILE}
  uses
    PCharLIB, FileTool;
  {$ENDIF}
{$ENDIF}

type

  { TErrorRec }
  TErrorRec = class(TObject)
    ErrorCode: Word;
    Next: TErrorRec;
    constructor Create(AErrorCode: Word);
    destructor Destroy; override;
    procedure TestEqual(AErrorCode: Word; AMsg: PChar);
    function Msg: PChar;
  end;

const
  errList: TErrorRec = nil;
  Profile: PChar = nil;
  TickStart: Longint = 0;

var
  TempDebugStr: PChar;

{$IFDEF WIN32}
{ Convert physical address to logical address }
function ConvertAddr(Address: Pointer): Pointer; assembler;
asm
	TEST    EAX,EAX         { Always convert nil to nil }
	JE      @@1
	SUB     EAX, $1000      { offset from code start; code start set by linker to $1000 }
@@1:
end;
function ReturnAddress: Pointer;
asm
	{MOV     EAX,[ESP + 4] codegen dependent}
	MOV     EAX,[EBP - 4]
end;
{$ELSE}
{ Convert physical address to logical address }
function ConvertAddr(Address: Pointer): Pointer; assembler;
asm
	MOV     AX,Address.Word[0]
	MOV     DX,Address.Word[2]
	MOV     CX,DX                   { Don't convert 0000:0000 }
	OR      CX,AX
	JE      @@1
	CMP     DX,0FFFFH               { Don't convert FFFF:xxxx }
	JE      @@1
	MOV     ES,DX
	MOV     DX,ES:Word[0]
@@1:
end;
function ReturnAddress: Pointer; assembler;
asm
	MOV     AX,[BP].Word[2]
	MOV     DX,[BP].Word[4]
end;
{$ENDIF WIN32}

{ TErrorRec }
constructor TErrorRec.Create(AErrorCode: Word);
begin
  inherited Create;
  ErrorCode := AErrorCode;
  Next := errList;
  errList := Self;
end;

destructor TErrorRec.Destroy;
begin
  errList := Next;
  inherited Destroy;
end;

procedure TErrorRec.TestEqual(AErrorCode: Word; AMsg: PChar);
begin
  if (ErrorCode = AErrorCode) then
    raise EDupCode.Create(Format('Duplicate error code: %d', [AErrorCode]))
  else
    if (StrComp(Msg, AMsg) = 0) then
      raise EDupStr.Create(Format('Duplicate Error String:'#10'%s', [AMsg]));
end;

function TErrorRec.Msg: PChar;
begin
  FillChar(TempDebugStr^, 256, #0);
  LoadString(HInstance, ErrorCode, TempDebugStr, 255);
  Result := TempDebugStr;
end;

function UnregisteredError: String;
begin
  Result := 'Unregistered error code.';
end;

function FindErrString(AErrorCode: Word): String;
var
  Temp: TErrorRec;
begin
  Result := '';
  Temp := errList;
  while (Temp <> nil) and (Result = '') do
  begin
    if Temp.ErrorCode = AErrorCode then
      Result := StrPas(Temp.Msg);
    Temp := Temp.Next;
  end;
  if Result = '' then
    Result := UnregisteredError;
end;

function FormatError(RetAddress: Pointer; Info: String): PChar;
begin
  FillChar(TempDebugStr^, 256, #0);
  if RetAddress = nil then
    Result := StrFmt(TempDebugStr, 'DBGTRACE: no address given for '#10'%s', [Info])
  else
    Result := StrFmt(TempDebugStr, 'Caller Address: %p'#10'%s', [ConvertAddr(RetAddress), Info]);
end;

{$IFDEF DBGTRACE}
procedure TraceInfo(WndOwner: HWND; RetAddress: Pointer; Info: String);
begin
  if RetAddress = nil then RetAddress := ReturnAddress;
  {$IFNDEF WIN32} {YET NOT TESTED IN WIN32}
  if WndOwner = 0 then
    WndOwner := HWND(Ptr(DSeg, $2E)^);
  {$ENDIF}
  MessageBox(WndOwner, FormatError(RetAddress, Info),
    'TRACE', MB_OK or MB_ICONINFORMATION or MB_TASKMODAL);
end;

procedure Trace(WndOwner: HWND; RetAddress: Pointer; ErrorCode: Integer);
begin
  if RetAddress = nil then RetAddress := ReturnAddress;
  TraceInfo(WndOwner, RetAddress, FindErrString(ErrorCode));
end;

procedure TraceFmt(WndOwner: HWND; RetAddress: Pointer; ErrorCode: Integer;
  const Args: array of const);
begin
  if RetAddress = nil then RetAddress := ReturnAddress;
  TraceInfo(WndOwner, RetAddress, Format(FindErrString(ErrorCode), Args));
end;

{$ELSE}
procedure TraceInfo(WndOwner: HWND; RetAddress: Pointer; Info: String);
begin
end;
procedure Trace(WndOwner: HWND; RetAddress: Pointer; ErrorCode: Integer);
begin
end;
procedure TraceFmt(WndOwner: HWND; RetAddress: Pointer; ErrorCode: Integer;
  const Args: array of const);
begin
end;
{$ENDIF}

procedure RegisterErrorCode(ErrorCode: Word);
var
  ER: TErrorRec;
  Msg: array [0..255] of Char;
begin
  FillChar(Msg, SizeOf(Msg), #0);
  if LoadString(HInstance, ErrorCode, Msg, 255) = 0 then
    TraceInfo(0, ReturnAddress, UnregisteredError)
  else
  begin
    ER := errList;
    while ER <> nil do
    try
      ER.TestEqual(ErrorCode, Msg);
      ER := ER.Next;
    except
      on E: Exception do
      begin
	FillChar(Msg, 256, #0);
	ER := nil;
	TraceInfo(0, ReturnAddress, E.Message);
      end;
    end;
    if StrLen(Msg) > 0 then
      ER := TErrorRec.Create(ErrorCode);
  end;
end;

procedure Assert(Assertion: Boolean; RetAddress: Pointer; AErrorCode: Word);
const
  SavedErrorCode: Word = 0;
  SaveMsg: PChar = nil;
begin
  if Assertion then Exit;
  if not Assigned(RetAddress) then
    RetAddress := ReturnAddress;
  ErrorAddr := RetAddress;
  SavedErrorCode := AErrorCode;
  SaveMsg := StrFmt(TempDebugStr, '%s', [FindErrString(SavedErrorCode)]);
  asm
     {$IFNDEF WIN32}
     mov sp,bp
     pop bp
     add sp,10
     {$ENDIF}
  end;
  raise EASSERT.Create(StrPas(FormatError(ErrorAddr, StrPas(SaveMsg))))
	at ErrorAddr;
end;

{$F+}
procedure Clean_errList;{$IFNDEF WIN32}far;{$ENDIF}
begin
  while errList <> nil do
    errList.Free;
  StrDispose(Profile);
  StrDispose(TempDebugStr);
end;
{$F-}

initialization
  TempDebugStr := StrAlloc(256);
  AddExitProc(Clean_errList);
end.

The information provided above is provided "AS IS" without warranty of any kind.
Lucian Radulescu disclaims all warranties, either express or implied, including
the warranties of merchantability and fitness for a particular purpose.

In no event shall Lucian Radulescu or his suppliers be liable for any damages
whatsoever including direct, indirect, incidental, consequential, loss of business
profits or special damages, even if Lucian Radulescu or his suppliers have been
advised of the possibility of such damages.

Some states do not allow the exclusion or limitation of liability for consequential
or incidental damages so the foregoing limitation may not apply.

(*
   The way this code works:
     you should have in the initialization section of unit desired
     to trace some stuff like:

initialization
  {$IFDEF DBGTRACE}
  RegisterDebuggStr(1);
  RegisterDebuggStr(2);
  {$ENDIF}
  ...
  {your stuff here}
end.

  where the numbers in brakets represents indexes in a resource file
  linked to your application with {$R filename}, with errors or other useful
  info to be displayed when debug. Make sure to include the resources file
  only when DBGTRACE is defined (keep code small if not debugging.)

A..............................................................
  Let's say you want to use Assert "kinda" procedures (Require, Ensure).
  Then, in your code you will have something like:

  begin
    {$IFDEF DBGTRACE}
    Require(precondition, ReturnAddress, MyErrorStringIndex);
    {$ENDIF}
    {...... your code from here}
  end;

or

  begin
    {...... your code here}
    {$IFDEF DBGTRACE}
    Ensure(postcondition, ReturnAddress, MyErrorStringIndex);
    {$ENDIF}
  end;

or

  begin
    {...... your code here}
    {$IFDEF DBGTRACE}
    Assert(middlecondition, ReturnAddress, MyErrorStringIndex);
    {$ENDIF}
    {...... your code here}
  end;


  You got it? Using Assert, Require or Ensure, the code raises
  exceptions if some vital-conditions are false. Now, what if
  you don't want exceptions, but just messageboxes instead, with some
  useful info, like the address of caller, and some string info?

B..............................................................

  You might use some code like this:

  function TForm1.Changed: Boolean;
  begin
    Result := FModified;
    {or some stuff here...}
    {$IFDEF DBGTRACE}
    if Result then
      Trace(Handle, ReturnAddress, ModifiedStringIndex)
    else
      Trace(Handle, ReturnAddress, NotModifiedStringIndex);
    {$ENDIF}
  end;

  when debugging, you will now what happend (your data was changed or not).
*)

(* scheme with 3 tries (cascade), for sensitive code.

  begin
    {$IFDEF DBGRAISE}
    Require(precondition, ReturnAddress, nnn);
    {$ELSE}{$IFDEF DBGTRACE}
    if not precondition then
      Trace(Handle, ReturnAddress, nnn);
    {$ENDIF}{$ENDIF}

    try
      try
        {try first algorithm}

        {$IFDEF DBGTRACE}
        if not condition1 then
          Trace(Handle, nil, nnn);
        {$ENDIF}

      except
        on E: ControlledException1 do
        try
          try
            {try second algorithm}

            {$IFDEF DBGTRACE}
            if not condition2 then
              Trace(Handle, nil, nnn);
            {$ENDIF}

          except
            on E: ControlledException2 do
            try
              try
                {try third algorithm}

                {$IFDEF DBGTRACE}
                if not condition3 then
                  Trace(Handle, nil, nnn);
                {$ENDIF}

              except
                on E: ControlledExceptionN do ;
                else raise;
              end;
            finally
              {free memory if necessary}
            end;
          end;
        finally
          {free memory if necessary}
        end;
      end;
    finally
      {free memory if necessary}
    end;

    {$IFDEF DBGRAISE}
    Ensure(postcondition, ReturnAddress, nnn);
    {$ELSE} {$IFDEF DBGTRACE}
    if not postcondition then
      Trace(Handle, ReturnAddress, nnn);
    {$ENDIF}{$ENDIF}
  end;

*)
