{*******************************************************}
{                                                       }
{       EQ_Abort                                        }
{       -----------------------------------------       }
{       ADV Compiled                                    }
{       Copyright (C) 1994, Lucian Radulescu            }
{       E-mail:    ljee@interlog.com                    }
{       Home Page: http://www.interlog.com/~ljee        }
{       Yes, 1994. This unit started with BP7WIN.       }
{       Used: file copy/compress operations,            }
{             heavy loops,                              }
{             abort boxes (print, copy, etc)            }
{*******************************************************}

unit EQ_Abort;

interface

uses
  {$IFDEF WIN32} Windows,
  {$ELSE}        WinTypes, WinProcs,
  {$ENDIF}       SysUtils;

type

  EUserAborted   = class(Exception);

function AbortProc: Boolean;
procedure DoIdle(MiliSeconds: LongInt; ForceOnePeek: Boolean);

var
  {$IFDEF DEMOCOPY}
  SlowCounter   : Integer;
  {$ENDIF DEMOCOPY}
  UserAbort     : Boolean;  {other code responsible for setting this accordingly}
  WantException : Boolean;  {set it to true, to raise EUserAborted when needed}
  UseAbortProc  : Boolean;  {just a global var I need somewhere...}

implementation

procedure DoIdle(MiliSeconds: LongInt; ForceOnePeek: Boolean);
var
  Msg: TMsg;
  StartTime: LongInt;
begin
  if (MiliSeconds = 0) and not ForceOnePeek then Exit;
  {$IFDEF WIN32}
  Sleep(MiliSeconds);
  if MiliSeconds > 0 then Exit;
  {$ENDIF WIN32}
  StartTime := GetTickCount;
  repeat
    while PeekMessage(Msg, 0, 0, 0, pm_Remove) do
    begin
      TranslateMessage(Msg);
      DispatchMessage(Msg);
    end;
  until (MiliSeconds = 0) or (Now - StartTime >= MiliSeconds) or (Now < StartTime);
end;

function AbortProc: Boolean;
begin
  Result := not UserAbort;
  if UserAbort and WantException then
    raise EUserAborted.Create('Operation cancelled');
  {$IFDEF DEMOCOPY}
  DoIdle(SlowCounter, False);
  {$ENDIF DEMOCOPY}
  DoIdle(0, True);  {at least one loop even if SlowCounter in use}
end;

initialization
  {$IFDEF DEMOCOPY}
  SlowCounter   := 5;
  {$ENDIF DEMOCOPY}
  UserAbort     := False;
  WantException := False;
  UseAbortProc  := False;
end. { END UNIT }

The information provided above is provided "AS IS" without warranty of any kind.
Lucian Radulescu disclaims all warranties, either express or implied, including
the warranties of merchantability and fitness for a particular purpose.

In no event shall Lucian Radulescu or his suppliers be liable for any damages
whatsoever including direct, indirect, incidental, consequential, loss of business
profits or special damages, even if Lucian Radulescu or his suppliers have been
advised of the possibility of such damages.

Some states do not allow the exclusion or limitation of liability for consequential
or incidental damages so the foregoing limitation may not apply.


