unit IconTool;

interface

uses
  {$IFDEF WIN32} Windows,
  {$ELSE}        WinTypes, WinProcs,
  {$ENDIF}
  ShellAPI, SysUtils, Classes, Controls, Forms, Graphics, ExtCtrls;

function  Icon2Bitmap(Icon: TIcon): TBitmap;
function  BitmapFromIcon(FileName: String): TBitmap;
procedure DrawDisabledBitmap(Canvas: TCanvas; X, Y: Integer; Bmp: TBitmap);
procedure DrawTransparentBitmap(Canvas: TCanvas; Bmp: TBitmap; xStart, yStart: Integer;
  cTransparentColor: LongInt);
{$IFDEF WIN32}
function Get_WC_Image(W: TWinControl; WhiteBckgr: Boolean): TBitmap;
{$ENDIF WIN32}

implementation

function Icon2Bitmap(Icon: TIcon): TBitmap;
begin
  Result        := TBitmap.Create;
  Result.Width  := Icon.Width;
  Result.Height := Icon.Height;
  Result.Canvas.Draw(0, 0, Icon);
end;

function AppIcon2Bitmap(AppPath: String): TBitmap;
var
  IconHandle : HICON;
  Icon       : TIcon;
  S          : Array [0..255] of Char;
begin
  Icon := TIcon.Create;
  IconHandle := ExtractIcon(HInstance, StrPCopy(S, AppPath), 0);
  if IconHandle = 0 then
    IconHandle := CopyIcon({$IFNDEF WIN32}HInstance,{$ENDIF}LoadIcon(0, IDI_APPLICATION));
  try
    Icon.Handle := IconHandle;
    Result := Icon2Bitmap(Icon);
  finally
    Icon.Free;
  end;
end;

function IconFile2Bitmap(IcoPath: String): TBitmap;
var
  Icon: TIcon;
begin
  Icon := TIcon.Create;
  Icon.LoadFromFile(IcoPath);
  try
    Result := Icon2Bitmap(Icon);
  finally
    Icon.Free;
  end;
end;

function BitmapFromIcon(FileName: String): TBitmap;
begin
  Result := nil;
  if pos('EXE', Uppercase(ExtractFileExt(FileName))) > 0 then
    Result := AppIcon2Bitmap(FileName)
  else
    if pos('ICO', Uppercase(ExtractFileExt(FileName))) > 0 then
      Result := IconFile2Bitmap(FileName);
end;

procedure DrawDisabledBitmap(Canvas: TCanvas; X, Y: Integer; Bmp: TBitmap);
var
  MonoBmp: TBitmap;
begin
  MonoBmp := TBitmap.Create;
  try
    MonoBmp.Assign(bmp);
    MonoBmp.Canvas.Brush.Color := clBlack;
    MonoBmp.Monochrome := True;
    Canvas.Brush.Color := clBtnHighlight;
    SetTextColor(Canvas.Handle, clBlack);
    SetBkColor(Canvas.Handle, clWhite);
    BitBlt(Canvas.Handle, x+1, y+1, bmp.Width, bmp.Height, MonoBmp.Canvas.Handle, 0, 0, $00E20746);
    Canvas.Brush.Color := clBtnShadow;
    SetTextColor(Canvas.Handle, clBlack);
    SetBkColor(Canvas.Handle, clWhite);
    BitBlt(Canvas.Handle, x, y, bmp.Width, bmp.Height, MonoBmp.Canvas.Handle, 0, 0, $00E20746);
  finally
    MonoBmp.Free;
  end
end;

procedure DrawTransparentBitmap(Canvas: TCanvas; Bmp: TBitmap; xStart, yStart: Integer;
  cTransparentColor: LongInt);
var
  {$IFDEF WIN32}
  bm: Windows.TBitmap;
  ptSize: Windows.TPoint;
  {$ELSE}
  bm: WinTypes.TBitmap;
  ptSize: WinTypes.TPoint;
  {$ENDIF}
  cColor: TColorRef;
  bmAndBack, bmAndObject, bmAndMem, bmSave, oldBmp: HBITMAP;
  bmBackOld, bmObjectOld, bmMemOld, bmSaveOld, hBmp: HBITMAP;
  hdcMem, hdcBack, hdcObject, hdcTemp, hdcSave, dc: HDC;
  temp_bitmap: TBitmap;
begin
  temp_bitmap := TBitmap.Create;
  temp_bitmap.Assign(bmp);
  try
    dc := canvas.Handle;
    hBmp := temp_bitmap.Handle;
    hdcTemp := CreateCompatibleDC(dc);
    oldBmp := SelectObject(hdcTemp, hBmp);

    GetObject(hBmp, SizeOf(bm), @bm);
    ptSize.x := bm.bmWidth;
    ptSize.y := bm.bmHeight;

    hdcBack   := CreateCompatibleDC(dc);
    hdcObject := CreateCompatibleDC(dc);
    hdcMem    := CreateCompatibleDC(dc);
    hdcSave   := CreateCompatibleDC(dc);

    bmAndBack   := CreateBitmap(ptSize.x, ptSize.y, 1, 1, nil);

    bmAndObject := CreateBitmap(ptSize.x, ptSize.y, 1, 1, nil);

    bmAndMem    := CreateCompatibleBitmap(dc, ptSize.x, ptSize.y);
    bmSave      := CreateCompatibleBitmap(dc, ptSize.x, ptSize.y);

    bmBackOld   := SelectObject(hdcBack, bmAndBack);
    bmObjectOld := SelectObject(hdcObject, bmAndObject);
    bmMemOld    := SelectObject(hdcMem, bmAndMem);
    bmSaveOld   := SelectObject(hdcSave, bmSave);

    SetMapMode(hdcTemp, GetMapMode(dc));

    BitBlt(hdcSave, 0, 0, ptSize.x, ptSize.y, hdcTemp, 0, 0, SRCCOPY);

    cColor := SetBkColor(hdcTemp, cTransparentColor);

    BitBlt(hdcObject, 0, 0, ptSize.x, ptSize.y, hdcTemp, 0, 0, SRCCOPY);

    SetBkColor(hdcTemp, cColor);

    BitBlt(hdcBack, 0, 0, ptSize.x, ptSize.y, hdcObject, 0, 0, NOTSRCCOPY);
    BitBlt(hdcMem, 0, 0, ptSize.x, ptSize.y, dc, xStart, yStart, SRCCOPY);
    BitBlt(hdcMem, 0, 0, ptSize.x, ptSize.y, hdcObject, 0, 0, SRCAND);
    BitBlt(hdcTemp, 0, 0, ptSize.x, ptSize.y, hdcBack, 0, 0, SRCAND);
    BitBlt(hdcMem, 0, 0, ptSize.x, ptSize.y, hdcTemp, 0, 0, SRCPAINT);
    BitBlt(dc, xStart, yStart, ptSize.x, ptSize.y, hdcMem, 0, 0, SRCCOPY);
    BitBlt(hdcTemp, 0, 0, ptSize.x, ptSize.y, hdcSave, 0, 0, SRCCOPY);

    DeleteObject(SelectObject(hdcBack, bmBackOld));
    DeleteObject(SelectObject(hdcObject, bmObjectOld));
    DeleteObject(SelectObject(hdcMem, bmMemOld));
    DeleteObject(SelectObject(hdcSave, bmSaveOld));

    SelectObject(hdcTemp, oldBmp);

    DeleteDC(hdcMem);
    DeleteDC(hdcBack);
    DeleteDC(hdcObject);
    DeleteDC(hdcSave);
    DeleteDC(hdcTemp);
  finally
    temp_bitmap.Free;
  end;
end;

{$IFDEF WIN32}
function Get_WC_Image(W: TWinControl; WhiteBckgr: Boolean): TBitmap;
var
  Ofs: Integer;
  R: TRect;
begin
  Result := TBitmap.Create;
  try
    Result.Width := W.ClientWidth;
    Result.Height := W.ClientHeight;
    if not WhiteBckgr then
    begin
      Result.Canvas.Brush := W.Brush;
      Result.Canvas.FillRect(W.ClientRect);
    end;
    Result.Canvas.Lock;
    try
      if GetWindowLong(W.Handle, GWL_STYLE) and WS_BORDER <> 0 then
	Ofs := -1     // Don't draw border
      else Ofs := 0;  // There is no border
      W.PaintTo(Result.Canvas.Handle, Ofs, Ofs);
      if WhiteBckgr then
      begin
	Result.Canvas.Brush.Color := clWhite;
	R := Rect(0, 0, Result.Width, Result.Height);
	Result.Canvas.BrushCopy(R, Result, R, Result.Canvas.Pixels[0, Result.Height-1]);
      end;
    finally
      Result.Canvas.Unlock;
    end;
  except
    Result.Free;
    raise;
  end;
end;
{$ENDIF WIN32}

end.

