unit LStrings;

interface

uses
  SysUtils, Classes;

type

  TLanguageLIBs = (tlEnglish, tlFrench);
  TStringsIndex = (tsiFirst, tsiLast); {this is going to be a very long list.
                                        I'm not sure if support more than 255 elements}

  Talfastr      = (taDlgCaps, taMenusCaps, taQueryStr); {etc}

var

  gStrings: TStringList;

function tsiStr(Index: TStringsIndex): String;
function tsiChr(Index: TStringsIndex): PChar;

implementation

var
  TempChar1, TempChar2: PChar;
  LastPChar: Boolean;

function tsiStr(Index: TStringsIndex): String;
begin
  Result := gStrings[ord(Index)];
end;

function tsiChr(Index: TStringsIndex): PChar;
begin
  if LastPChar then
  begin
    StrPCopy(TempChar1, tsiStr(Index));
    Result := TempChar1;
  end
  else
  begin
    StrPCopy(TempChar2, tsiStr(Index));
    Result := TempChar2;
  end;
  LastPChar := not LastPChar;
end;

function LoadStringLIB(ID_LANGUAGE: TLanguageLIBs): Boolean;
const
  StartResID = JEE_ID;
var
  LoopStrings: TStringsIndex;
  DLL_Handle: THandle;                                   { DLL handle }
begin
  Result := False;                                       { assume failed }
  case ID_LANGUAGE of                                    { select language }
    tlEnglish : StrCopy(TempChar1, 'STRSENG.DLL');       { english }
    tlFrench  : StrCopy(TempChar1, 'STRSFRE.DLL');       { french }
    else Exit;  {imposible statement}                    { error, exit, no changes }
  end;

  if Assigned(gStrings) then gStrings.Free;              { free old string list }
  gStrings := nil;

  DLL_Handle := LoadLibrary(StrPas(TempChar1));          { load desired library }
  if DLL_Handle <= HINSTANCE_ERROR then                  { check for success }
    MessageBox(0, StrCat(StrCopy(TempChar2, 'Unable to load library '), TempChar1),
                  'Fatal', MB_ICONSTOP or MB_OK or MB_TASKMODAL)
  else
  try
    for LoopStrings := Low(TStringsIndex) to High(TStringsIndex) do
      if LoadString(LibHandle, StartResID + ord(LoopStrings), TempChar1, 255) > 0 then
      begin
        if not Assigned(gStrings) then                   { if not assigned yet }
          gStrings := TStringList.Create;                { create new string list }
        gStrings.Add(LoadStr(StrPas(TempChar1))));       { add string to list }
      end;
    Result := True;                                      { success }
  finally
    FreeLibrary(LibHandle);                              { free library }
  end;
end;

procedure Destroy_LStrings; far;
begin
  StrDispose(TempChar1);
  StrDispose(TempChar2);
  if Assigned(gStrings) then gStrings.Free;
end;

initialization
  TempChar1 := StrAlloc(256);
  TempChar2 := StrAlloc(256);
  FillChar(TempChar1^, 256, #0);
  FillChar(TempChar2^, 256, #0);
  AddExitProc(Destroy_LStrings);
  LastPChar := True;
  gStrings := nil;
  if not LoadStringLIB(tlEnglish) then
    Halt(1);
end.

