{***********************************************************}
{                                                           }
{     MemIniF                                               }
{     ============                                          }
{     ADV Compiled                                          }
{     Copyright  1995-1998 by Lucian Radulescu             }
{     E-mail    : ljee@interlog.com                         }
{     Home Page : http://www.interlog.com/~ljee             }
{                                                           }
{     File status: FREEWARE                                 }
{                  (you may use it, but not sell it)        }
{     Last time reviewed: May 15th, 1998.                   }
{                                                           }
{***********************************************************}

{$I ALLLIB.DEF}

unit MemIniF;

interface

uses SysUtils, Classes;

type

  { TMemoryIniFile }
  TMemoryIniFile = class(TStringList)
  private
    { Private declarations }

    FIndex   : Integer;
    FSection : String;
    function  IsSection(const Position: Integer): Boolean;
    function  Locate(const Ident: String): Integer;
    procedure SetSection(const Value: String);

  protected
    { Protected declarations }

    property Cur: Integer read FIndex;

  public
    { Public declarations }

    constructor Create;
    function  ReadString(const ASection, Ident, Default: string): string;
    function  ReadInteger(const ASection, Ident: string; Default: Longint): Longint;
    function  ReadBool(const ASection, Ident: string; Default: Boolean): Boolean;
    procedure ReadSection(const ASection: string; AStrings: TStrings);
    procedure ReadSectionValues(const ASection: string; Strings: TStrings);
    procedure WriteString(const ASection, Ident, Value: String);
    procedure WriteInteger(const ASection, Ident: string; Value: Longint);
    procedure WriteBool(const ASection, Ident: string; Value: Boolean);
    procedure EraseSection(const ASection: string);

    property Section: String read FSection write SetSection;

  end;

implementation

{ TMemoryIniFile }
constructor TMemoryIniFile.Create;
begin
  inherited Create;
  Duplicates := dupAccept;
  FSection := '';
end;

function TMemoryIniFile.IsSection(const Position: Integer): Boolean;
var
  S: String;
begin
  Result := (Position <> -1) and (Position < Count);
  if Result then
  begin
    S := Strings[Position];
    Result := (Pos('[', S) = 1) and (Pos(']', S) = Length(S));
  end;
end;

function TMemoryIniFile.Locate(const Ident: String): Integer;
var
  SearchIt: String;
begin
  SearchIt := Format('%s=', [Ident]);
  Result := Cur + 1;
  if Result < Count then
  repeat
    if Pos(SearchIt, Strings[Result]) = 1 then Exit
    else Inc(Result);
  until (Result = Count) or IsSection(Result);
  Result := -1;
end;

procedure TMemoryIniFile.SetSection(const Value: String);
begin
  FSection := Format('[%s]', [Value]);
  FIndex := IndexOf(FSection);
end;

function TMemoryIniFile.ReadString(const ASection, Ident, Default: string): string;
var
  Loc: Integer;
  FindIt: String;
begin
  Section := ASection;
  if Section = '' then Result := Default
  else
  begin
    Loc := Locate(Ident);
    if Loc = -1 then Result := Default
    else
    begin
      FindIt := Strings[Loc];
      Result := System.Copy(FindIt, pos('=', FindIt)+1, 255);
    end;
  end;
end;

function TMemoryIniFile.ReadInteger(const ASection, Ident: string; Default: Longint): Longint;
var
  IStr: string;
begin
  IStr := ReadString(Section, Ident, '');
  if CompareText(Copy(IStr, 1, 2), '0x') = 0 then
    IStr := '$' + Copy(IStr, 3, 255);
  Result := StrToIntDef(IStr, Default);
end;

function TMemoryIniFile.ReadBool(const ASection, Ident: string; Default: Boolean): Boolean;
begin
  Result := ReadInteger(Section, Ident, Ord(Default)) <> 0;
end;

procedure TMemoryIniFile.ReadSection(const ASection: string; AStrings: TStrings);
var
  I: Integer;
begin
  Section := ASection;
  if Section = '' then Exit;
  I := Cur + 1;
  while (I < Count) and not IsSection(I) do
  begin
    AStrings.Add(Strings[I]);
    Inc(I)
  end;
end;

procedure TMemoryIniFile.ReadSectionValues(const ASection: string; Strings: TStrings);
var
  KeyList: TStringList;
  I: Integer;
begin
  KeyList := TStringList.Create;
  try
    ReadSection(Section, KeyList);
    for I := 0 to Keylist.Count - 1 do
      Strings.Values[KeyList[I]] := ReadString(Section, KeyList[I], '');
  finally
    KeyList.Free;
  end;
end;

procedure TMemoryIniFile.WriteString(const ASection, Ident, Value: String);
var
  Loc: Integer;
begin
  Section := ASection;
  if Cur = -1 then
  begin
    Add(Section);
    FIndex := Pred(Count);
  end;
  Loc := Locate(Ident);
  if Loc = -1 then
    Insert(Succ(Cur), Format('%s=%s', [Ident, Value]))
  else
  begin
    Delete(Loc);
    Insert(Loc, Format('%s=%s', [Ident, Value]));
  end;
end;

procedure TMemoryIniFile.WriteInteger(const ASection, Ident: string; Value: Longint);
begin
  WriteString(ASection, Ident, IntToStr(Value));
end;

procedure TMemoryIniFile.WriteBool(const ASection, Ident: string; Value: Boolean);
const
  Vals: array [Boolean] of string[1] = ('0', '1');
begin
  WriteString(ASection, Ident, Vals[Value]);
end;

procedure TMemoryIniFile.EraseSection(const ASection: string);
begin
  Section := ASection;
  if Cur = -1 then Exit;
  repeat
    Delete(Cur);
  until (Cur = Count) or IsSection(Cur);
end;

end.

The information provided above is provided "AS IS" without warranty of any kind.
Lucian Radulescu disclaims all warranties, either express or implied, including
the warranties of merchantability and fitness for a particular purpose.

In no event shall Lucian Radulescu or his suppliers be liable for any damages
whatsoever including direct, indirect, incidental, consequential, loss of business
profits or special damages, even if Lucian Radulescu or his suppliers have been
advised of the possibility of such damages.

Some states do not allow the exclusion or limitation of liability for consequential
or incidental damages so the foregoing limitation may not apply.

