{*******************************************************}
{                                                       }
{       PChars - Implementation  Part                   }
{       -----------------------------                   }
{       ADV Compiled                                    }
{       Copyright (C), Lucian Radulescu                 }
{       E-mail:    ljee@interlog.com                    }
{       Home Page: http://www.interlog.com/~ljee        }
{                                                       }
{*******************************************************}

unit PCharLIB;

{$I PCHARLIB.INT}

implementation

{$IFNDEF VER100}
{$IFDEF WIN32}
function CompareMem(P1, P2: Pointer; Length: Integer): Boolean; assembler;
asm
        PUSH    ESI
        PUSH    EDI
        MOV     ESI,P1
        MOV     EDI,P2
        MOV     EDX,ECX
        XOR     EAX,EAX
        AND     EDX,3
        SHR     ECX,2
        REPE    CMPSD
        JNE     @@2
        MOV     ECX,EDX
        REPE    CMPSB
        JNE     @@2
@@1:    INC     EAX
@@2:    POP     EDI
        POP     ESI
end;
{$ELSE}
function CompareMem(P1, P2: Pointer; Length: Integer): Boolean; assembler;
asm
        PUSH DS         { Save DS }
        MOV  AX, 1      { Result: True }
        MOV  CX, Length { Bufer size in CX }
        JCXZ @@DONE     { Finnished if size is zero }
        LDS  SI, P1     { DS:SI points to P1 }
        LES  DI, P2     { ES:DI points to P2 }
        CLD             { Flag to bump SI, DI }
        REP  CMPSB      { compare buffers }
        JE   @@Done     { done if equals }
        XOR  AX, AX     { Result: False }
@@DONE: POP  DS	        { Restore DS }
end;
{$ENDIF}
{$ENDIF VER100}

function StrNewPas(S: String): PChar;
begin
  if (Length(S) = 0) or (S = #0) then Result := nil
  else
    Result := StrPCopy(StrAlloc(Length(S)+1), S);
end;

function ReplacePChar(sOld, sNew: PChar): PChar;
begin
  Result := sNew;
  if Assigned(sNew) then
    if not Assigned(sOld) then Exit
    else
      if StrComp(sOld, sNew) = 0 then Result := sOld
      else MyStrDispose(sOld);
end;

{$IFNDEF WIN32}
procedure SetLength(var S: string; NewLength: Integer);
begin
  S[0] := chr(NewLength);
end;
procedure SetString(var S: string; Buffer: PChar; Len: Integer);
begin
  Move(Buffer^, S[1], Len);
  S[0] := chr(Len);
end;
{$ENDIF}

function NilPtr(P: PChar): Boolean;
begin
  Result := (P = nil) or (P^ = #0);
end;

procedure MyStrDispose(var P: PChar);
begin
  if P <> nil then
  begin
    Dec(P, SizeOf(Cardinal));
    FreeMem(P, Cardinal(Pointer(P)^));
  end;
  P := nil;
end;

function GetClearStr(Size: Cardinal): PChar;
begin
  Result := StrAlloc(Size);    { Allocate size }
  FillChar(Result^, Size, #0); { stralloc doesn't fill with zeroes }
end;

function EmptyPChar(S: PChar; Size: Cardinal): PChar;
begin
  Result := S;
  FillChar(S^, Size, #0);
end;

function ScanStr(Str: PChar; Chr: Char): PChar;
begin
  Result := Str;
  if NilPtr(Result) then Exit;
  repeat
    if Result^ = Chr then Exit
    else Inc(Result);
  until (Result^ = #0);
  Result := nil;
end;

procedure StrScanCopy(DestLeft, DestRight, Source: PChar; Chr: Char);
var
  P: PChar;
begin
  P := ScanStr(Source, Chr);
  if P = nil then StrCopy(DestLeft, Source)
  else
  begin
    StrLCopy(DestLeft, Source, P - Source);
    Inc(P);
    StrCopy(DestRight, P);
  end;
end;

function StrScanOrFirst(Str: PChar; Chr: Char): PChar;
begin
  Result := ScanStr(Str, Chr);
  if Result = nil then
    Result := Str;
end;

function StrTok(var Buff: PChar; Str: PChar): PChar;
var
  SOk, P: PChar;
  L: Cardinal;
begin
  Result := nil;
  if NilPtr(Buff) or NilPtr(Str) then Exit
  else
  begin
    P := StrPos(Buff, Str);
    if NilPtr(P) then Exit;
    L := P - Buff;
    if (L = 0) then  {"Buff" starts with searched token "str"}
      Exit;
    SOk := GetClearStr(L + 1);
    if SOk = nil then Exit;
    StrLCopy(SOk, Buff, L);
    Buff := P;
    Result := SOk;
  end;
end;

function StrTokOr(var Buff: PChar; Str: PChar): PChar;
begin
  Result := StrTok(Buff, Str);
  if Result = nil then
  begin
    Result := StrDup(Buff);
    Buff := nil;
  end
  else
    Inc(Buff, StrLen(Str));
end;

function DeleteStr(S: PChar; Start, Count: Cardinal): PChar;
var
  P, T: PChar;
  Len: Cardinal;
begin
  P := nil;
  if not NilPtr(S) then
  begin
    Len := StrLen(S);
    if (Start <= Len) and (Count > 0) then
    begin
      Dec(Len, Start);
      Dec(Start);
      T := S;
      Inc(T, Start + Count);
      P := GetClearStr(Len);
      StrCat(StrLCopy(P, S, Start), T);
    end;
  end;
  Result := P;
end;

function DeletePChar(Buff: PChar; Str: PChar; All: Boolean): PChar;
var
  P, SOk, Ofst, Tmp, TmpBuff: PChar;
  L, N: Cardinal;
begin
  Result := Buff;
  if NilPtr(Buff) or NilPtr(Str) then
    Exit;

  L := StrLen(Buff) - StrLen(Str);
  if L = 0 then
    if StrComp(Buff, Str) = 0 then
    begin
      Result := nil;
      Exit;
    end;

  P := StrPos(Buff, Str);
  if NilPtr(P) then
    Exit;

  SOk := GetClearStr(L + 1);
  if SOk = nil then Exit
  else Ofst := SOk;

  TmpBuff := Buff;
  N := P-TmpBuff;
  StrMove(Ofst, TmpBuff, N);
  Inc(Ofst, N);
  Inc(TmpBuff, N+StrLen(Str));
  StrMove(Ofst, TmpBuff, L-N);

  if All then
  begin
    P := StrPos(SOk, Str);
    if not NilPtr(P) then
    begin
      Tmp := DeletePChar(SOk, Str, All);
      if not NilPtr(Tmp) then
      begin
        MyStrDispose(SOk);
        Result := Tmp;
        Exit;
      end;
    end;
  end;

  Result := SOk;
end;

function ScanDelete(Buff: PChar; Ch: Char): PChar;
var
  Q: PChar;
begin
  Result := Buff;
  if NilPtr(Buff) or (Ch = #0) then Exit;
  Q := Buff;
  repeat
    Q := ScanStr(Q, Ch);
    if NilPtr(Q) then Break
    else StrMove(Q, Q+1, StrEnd(Q)-Q);
  until NilPtr(Q);
end;

function ScanReplace(Buff: PChar; ChSearch, ChReplace: Char): PChar;
var
  Q: PChar;
begin
  Result := Buff;
  if NilPtr(Buff) or (ChSearch = #0) then Exit;
  if ChReplace = #0 then Result := ScanDelete(Buff, ChSearch)
  else
  begin
    Q := Buff;
    repeat
      Q := ScanStr(Q, ChSearch);
      if NilPtr(Q) then Break
      else Q^ := ChReplace;
    until NilPtr(Q);
  end;
end;

function ExtractPChar(var Buff: PChar; FromStr, ToStr: PChar): PChar;
var
  L, R, S: PChar;
  LenFrom, Len: Cardinal;
begin
  Result := nil;
  if not NilPtr(Buff) then
  begin
    L := StrPos(Buff, FromStr);
    R := StrPos(Buff, ToStr);
    if not NilPtr(L) and not NilPtr(R) then
    begin
      LenFrom := StrLen(FromStr);
      Len := R-L-LenFrom+1;
      S := GetClearStr(Len);
      if S = nil then Exit;
      Inc(L, LenFrom);
      StrLCopy(S, L, Len-1);
      Result := S;
      Inc(R, StrLen(ToStr));
    end;
    Buff := R;
  end;
end;

function ExtractPCharEx(var Buff: PChar; FromStr, ToStr: PChar): PChar;
var
  L, R, S: PChar;
  LenFrom, Len: Cardinal;
begin
  Result := nil;
  if not NilPtr(Buff) then
  begin
    L := StrPos(Buff, FromStr);
    R := StrPos(Buff, ToStr);
    if not NilPtr(L) and (R = L) then
    begin
      Inc(R);
      R := StrPos(R, ToStr);
    end;
    if not NilPtr(L) and not NilPtr(R) then
    begin
      LenFrom := StrLen(FromStr);
      Len := R-L-LenFrom+1;
      S := GetClearStr(Len);
      if S = nil then Exit;
      Inc(L, LenFrom);
      StrLCopy(S, L, Len-1);
      Result := S;
      Inc(R, StrLen(ToStr));
    end;
    Buff := R;
  end;
end;

function ExtractSubstr(var Buff: PChar; FromStr, ToStr: PChar): PChar;
var
  S: PChar;
  Len: Cardinal;
begin
  Result := nil;
  if not NilPtr(Buff) then
  begin
    if not NilPtr(FromStr) and not NilPtr(ToStr) then
    begin
      Len := ToStr-FromStr;
      S := GetClearStr(Len);
      if S = nil then Exit;
      StrLCopy(S, FromStr+1, Len-1);
      Result := S;
      Inc(ToStr);
    end;
    Buff := ToStr;
  end;
end;

function ExtractFullSubstr(var Buff: PChar; FromStr, ToStr: PChar): PChar;
var
  S: PChar;
  Len: Cardinal;
begin
  Result := nil;
  if not NilPtr(Buff) then
  begin
    if not NilPtr(FromStr) and not NilPtr(ToStr) then
    begin
      Inc(ToStr);
      Len := ToStr-FromStr + 1;
      S := GetClearStr(Len);
      if S = nil then Exit;
      StrLCopy(S, FromStr, Len-1);
      Result := S;
    end;
    Buff := ToStr;
  end;
end;

procedure ReplaceStr(var Buff: PChar; Search, Replace: PChar);
var
  P, S: PChar;
begin
  if not NilPtr(Buff) and not NilPtr(Search) and not NilPtr(Replace) then
  begin
    S := nil;
    P := Buff;
    while StrPos(P, Search) <> nil do
    begin
      AddLToPChar(S, P, StrPos(P, Search)-P);
      AddToPChar(S, Replace);
      P := StrPos(P, Search);
      if not NilPtr(P) then
        Inc(P, StrLen(Search));
    end;
    if not NilPtr(P) then
      AddToPChar(S, P);
    MyStrDispose(Buff);
    Buff := S;
  end;
end;

function ReplaceSubstr(var Buff: PChar; FromStr, ToStr, Replace: PChar): PChar;
var
  S: PChar;
  Len, L: Cardinal;
begin
  Result := nil;
  if not NilPtr(Buff) then
  begin
    if not NilPtr(FromStr) and not NilPtr(ToStr) then
    begin
      Len := StrLen(Buff)-(ToStr-FromStr)+StrLen(Replace);
      S := GetClearStr(Len);
      if S = nil then Exit;
      L := FromStr-Buff;
      Inc(ToStr);
      StrCat(StrCat(StrLCopy(S, Buff, L), Replace), ToStr);
      MyStrDispose(Buff);
      Buff := S;
      Result := Buff;
    end;
  end;
end;

function ClearCRLF(Buff: PChar): PChar;
var
  P: PChar;
begin
  Result := nil;
  if NilPtr(Buff) then Exit;
  P := Buff;                              { Working string    }
  Result := DeletePChar(P, #13#10, True); { Delete All <CRLF> }
end;

function StripEndNonCh(Buff: PChar): PChar; {Aug. 10, 1999}
var
  P: PChar;
begin
  Result := Buff;
  if NilPtr(Result) then Exit;
  P := StrEnd(Buff);
  repeat
    Dec(P);
    if P^ < #32 then P^ := #0;
  until (P^ >= #32);
  Result := Buff;
end;

function RetrieveAfter(Buff, Str: PChar): PChar;
var
  S, P: PChar;
  Len: Cardinal;
begin
  Result := nil;
  if NilPtr(Buff) or NilPtr(Str) then Exit;
  P := StrPos(Buff, Str);                 { Find First Str }
  Len := StrLen(P);                       { Length of P    }
  S := GetClearStr(Len+1);                { Allocate S }
  StrMove(S, P, Len);                     { Copy Str to S  }
  Result := S;                            { Return S       }
end;

function RetrieveAfterSkip(Buff, Str: PChar): PChar;
var
  S, P: PChar;
  Len: Cardinal;
begin
  Result := nil;
  if NilPtr(Buff) or NilPtr(Str) then Exit;
  P := StrPos(Buff, Str);                 { Find First Str }
  Inc(P, StrLen(Str));                    { Skip Str       }
  Len := StrLen(P);                       { Length of P    }
  S := GetClearStr(Len+1);                { Allocate S }
  StrMove(S, P, Len);                     { Copy Str to S  }
  Result := S;                            { Return S       }
end;

function RetrieveAfterSkipFree(var Buff: PChar; Str: PChar): PChar;
begin
  Result := RetrieveAfterSkip(Buff, Str);
  StrDispose(Buff);
  Buff := nil;
end;

function StrCopyFreeSource(Buff, Source: PChar): PChar;
begin
  Result := Buff;
  if NilPtr(Source) then Exit;
  try
    StrCopy(Result, Source);
  finally
    MyStrDispose(Source);
  end;
end;

function RevertPChar(Buff: PChar; Size: Cardinal): PChar;
var
  P, S, T: PChar;
begin
  T := Buff;
  S := GetClearStr(Size+1);
  P := S;
  Inc(Buff, StrLen(Buff));
  while Buff > T do
  begin
    Dec(Buff);
    S^ := Buff^;
    Inc(S);
  end;
  S^ := #0;
  Result := P;
end;

function RevertPChar1(Buff: PChar; Size: Cardinal): PChar;
begin
  Result := RevertPChar(Buff, Size-1);
end;

function StrDup(Str: PChar): PChar;
var
  Len: Cardinal;
begin
  Result := nil;
  if NilPtr(Str) then Exit;
  Len := StrLen(Str);
  Result := StrLCopy(GetClearStr(Len+1), Str, Len);
end;

function StrPasEx(Str: PChar): String;
begin
  if NilPtr(Str) then Result := ''
  else
    Result := StrPas(Str);
end;

function StrLDup(Str: PChar; Count: Cardinal): PChar;
var
  Len: Cardinal;
begin
  Result := Str;
  if NilPtr(Str) then Exit;
  Len := StrLen(Str);
  if Count > Len then
    Count := Len;
  Result := StrLCopy(GetClearStr(Count+1), Str, Count);
end;

function StrPDup(Str: String): PChar;
var
  Len: Cardinal;
begin
  Result := nil;
  Len := Length(Str);
  if (Len=0) or (Str=#0) then Exit;
  Result := StrLCopy(StrAlloc(Len+1), @Str[1], Len);
end;

function StrNew_Test(Test: Boolean; StrYes, StrNo: PChar): PChar;
begin
  if Test then Result := StrDup(StrYes)
  else Result := StrDup(StrNo);
end;

function TrimBlanks(P: PChar): PChar;
var
  S: PChar;
begin
  Result := nil;
  if NilPtr(P) then Exit;
  S := P;
  while S^ = #32 do Inc(S);
  if StrLen(S) > 0 then
  begin
    while PChar(S+StrLen(S)-1)^ = #32 do
      PChar(S+StrLen(S)-1)^ := #0;
    if not NilPtr(S) then
      Result := StrDup(S);
  end;
  MyStrDispose(P);
end;

function TrimBlanks_Tabs(P: PChar): PChar;
var
  S: PChar;
begin
  Result := nil;
  if NilPtr(P) then Exit;
  S := P;
  while (S^ = #32) or (S^=#9) do Inc(S);
  if StrLen(S) > 0 then
  begin
    while (PChar(S+StrLen(S)-1)^ = #32) or (PChar(S+StrLen(S)-1)^ = #9) do
      PChar(S+StrLen(S)-1)^ := #0;
    if not NilPtr(S) then
      Result := StrDup(S);
  end;
  MyStrDispose(P);
end;

function TrimBlanks_TabsCRLF(P: PChar): PChar;
var
  S: PChar;
  Changed: Boolean;
begin
  Result := nil;
  if NilPtr(P) then Exit;
  S := P;
  repeat
    Changed := False;
    while StrPos(S, #13#10) = S do
    begin
      Inc(S, 2);
      Changed := True;
    end;
    while (S^ = #32) or (S^=#9) do
    begin
      Inc(S);
      Changed := True;
    end;
  until not Changed;
  if StrLen(S) > 0 then
  begin
    repeat
      Changed := False;
      while (PChar(S+StrLen(S)-1)^ = #32) or (PChar(S+StrLen(S)-1)^ = #9) do
      begin
        PChar(S+StrLen(S)-1)^ := #0;
        Changed := True;
      end;
      while StrPos(PChar(S+StrLen(S)-2), #13#10) <> nil do
      begin
        PChar(S+StrLen(S)-2)^ := #0;
        Changed := True;
      end;
    until not Changed;

    if not NilPtr(S) then
      Result := StrDup(S);
  end;
  MyStrDispose(P);
end;

function TrimPChar(P: PChar; Ch: CharSet): PChar;
var
  S: PChar;
begin
  Result := nil;
  if NilPtr(P) then Exit;
  S := P;
  while (S^ in Ch) do Inc(S);
  if StrLen(S) > 0 then
  begin
    while (PChar(S+StrLen(S)-1)^ in Ch) do
      PChar(S+StrLen(S)-1)^ := #0;
    if not NilPtr(S) then
      Result := StrDup(S);
  end;
  MyStrDispose(P);
end;

function GetWords(Line: PChar): Longint;

function MinPos(A, B: PChar): PChar;
begin
  if A < B then Result := A
  else Result := B;
end;

var
  S, B, T: PChar;
begin
  Result := 1;
  S := Line;
  while not NilPtr(S) do
  begin
    B := StrPos(S, #32);
    T := StrPos(S, #9);
    if (B <> nil) and (T <> nil) then S := MinPos(B, T) {if both not nil, pick minum}
    else
      if (B <> nil) and (T = nil) then S := B
      else
        if (B = nil) and (T <> nil) then S := T
        else Exit;
    Inc(S);
    Inc(Result);
  end;
end;

function StrNewOld(var Old: PChar; Str: PChar): PChar;
begin
  Result := nil;
  if not NilPtr(Old) and not NilPtr(Str) then
    if StrComp(Old, Str) = 0 then Exit;
  MyStrDispose(Old);
  Old := StrDup(Str);
  Result := Old;
end;

function StrINewOld(var Old: PChar; Str: PChar): PChar;
begin
  Result := nil;
  if not NilPtr(Old) and not NilPtr(Str) then
    if StrIComp(Old, Str) = 0 then Exit;
  MyStrDispose(Old);
  Old := StrDup(Str);
  Result := Old;
end;

{function StrPosMatch(Buff: PChar; Str: PChar; MatchCase, MatchWord, Forwrd: Boolean): PChar;

function GetP(Temp: PChar): PChar;
var
  UpS, LoS: PChar;
begin
  if MatchCase then Result := ScanStr(Temp, Str^)
  else
  begin
    if UpCase(Str^) = Str^ then
    begin
      LoS := ScanStr(Temp, Char(byte(Str^)+32));
      UpS := ScanStr(Temp, Str^);
    end
    else
    begin
      LoS := ScanStr(Temp, Str^);
      UpS := ScanStr(Temp, UpCase(Str^));
    end;
    if LoS < UpS then Result := LoS
    else Result := UpS;
  end;
end;

var
  P, Temp: PChar;
  Len: Cardinal;
  Ok: Boolean;
begin
  Result := nil;
  if NilPtr(Buff) or NilPtr(Str) then Exit;
  Len := StrLen(Str);
  Temp := Buff;
  repeat
    Ok := False;
    if Forwrd then P := GetP(Temp)
    else
    begin
      P := nil;
      if PtrRec(Temp).Ofs > 2 then Dec(Temp);
      if PtrRec(Temp).Ofs > 2 then
      repeat
        Dec(Temp);
        P := GetP(Temp);
      until (PtrRec(Temp).Ofs = 2) or ((P <> nil) and (P = Temp));
      if P > Temp then P := nil;
    end;
    if P <> nil then
    begin
      if MatchCase then Ok := StrLComp(Str, P, Len) = 0
      else Ok := StrLIComp(Str, P, Len) = 0;
      if Ok then
        if MatchWord then Ok := not (P[Len] in ['0'..'9', 'A'..'Z', 'a'..'z'])
        else Ok := True;
    end;
    if not Ok then
      if not NilPtr(P) then
        Temp := P+1
      else Temp := nil;
  until Ok or (Temp = nil);
  if Ok then
    Result := P;
end;
}
function CountChar(P: PChar; Ch: Char): Longint;
var
  S: PChar;
begin
  Result := 0;
  S := P;
  while not NilPtr(S) do
  begin
    S := ScanStr(S, Ch);
    if not NilPtr(S) then
    begin
      Inc(S);
      Inc(Result);
    end;
  end;
end;

function  StrCountChar(S: String; Ch: Char): Longint;
var
  K: Cardinal;
begin
  Result := 0;
  while S <> '' do
  begin
    K := Pos(Ch, S);
    if K = 0 then Break
    else Inc(Result);
    Delete(S, 1, K);
  end;
end;

function CountStr(P, Substr: PChar): Longint;
var
  S: PChar;
begin
  Result := 0;
  S := P;
  while not NilPtr(S) do
  begin
    S := StrPos(S, Substr);
    if not NilPtr(S) then
    begin
      Inc(S);
      Inc(Result);
    end;
  end;
end;

function GetParagraph(var Buff: PChar): PChar;
const
  CRLF: PChar = #13#10#0;
var
  Paragraph: PChar;
begin
  Paragraph := StrTok(Buff, CRLF);
  if NilPtr(Paragraph) and not NilPtr(Buff) then
  begin
    Paragraph := StrNew_Test(Buff = StrPos(Buff, CRLF), CRLF, Buff);
    Inc(Buff, StrLen(Paragraph));
  end
  else Inc(Buff, 2);
  Result := Paragraph;
end;

function IsEndLine(Line: PChar): Boolean;
const
  CRLF: PChar = #13#10#0;
begin
  Result := (StrComp(Line, CRLF) = 0) or
            (StrPos(Line, CRLF) = (StrEnd(Line) - 2));
end;

procedure AddToBuff(var Buff: PChar; S: String);
var
  Size, Len: Cardinal;
  P: PChar;
begin
  Len := Length(S);
  if Len = 0 then Exit;
  if NilPtr(Buff) then Size := Len
  else Size := Len+StrLen(Buff);
  P := GetClearStr(Size+1);
  if not NilPtr(Buff) then
  begin
    StrCopy(P, Buff);
    MyStrDispose(Buff);
  end;
  StrPCopy(@P[Size-Len], S);
  Buff := P;
end;

function AddToBuffFunc(Buff: PChar; S: String): PChar;
begin
  AddToBuff(Buff, S);
  Result := Buff;
end;

procedure AddLToPChar(var Buff: PChar; AddS: PChar; Count: Cardinal);
var
  P: PChar;
  Len, Size: Cardinal;
begin
  if NilPtr(AddS) or (Count = 0) then Exit;
  Len := Count;
  if Len > StrLen(Adds) then
    Len := StrLen(Adds);

  if NilPtr(Buff) then Size := Len
  else Size := Len+StrLen(Buff);
  P := GetClearStr(Size+1);
  if NilPtr(Buff) then StrLCopy(P, AddS, Len)
  else
  begin
    StrMove(StrEnd(StrCopy(P, Buff)), AddS, Len);
    MyStrDispose(Buff);
  end;
  Buff := P;
end;

procedure AddToPChar(var Buff: PChar; AddS: PChar);
var
  P: PChar;
  Len, Size: Cardinal;
begin
  if NilPtr(AddS) then Exit;
  Len := StrLen(Adds);
  if NilPtr(Buff) then Size := Len
  else Size := Len+StrLen(Buff);
  P := GetClearStr(Size+1);
  if NilPtr(Buff) then StrCopy(P, AddS)
  else
  begin
    StrCat(StrCopy(P, Buff), AddS);
    MyStrDispose(Buff);
  end;
  Buff := P;
end;

procedure AddPCharToBuff(var Buff: PChar; AddS: PChar);
var
  P: PChar;
  Len, Size: Cardinal;
begin
  if NilPtr(AddS) then Exit;
  Len := StrLen(Adds);
  if NilPtr(Buff) then Size := Len
  else Size := Len+StrLen(Buff)+1;
  P := GetClearStr(Size+1);
  if NilPtr(Buff) then StrCopy(P, AddS)
  else
  begin
    StrCat(StrCat(StrCopy(P, Buff), ' '), AddS);
    MyStrDispose(Buff);
  end;
  Buff := P;
end;

procedure AddToBuffDelim(var Buff: PChar; S, Delim: String);
var
  P: PChar;
  Size: Cardinal;
begin
  if S = '' then Exit;
  if Delim = '' then Exit; {wrong call. should use concat}
  Size := Length(S);
  if not NilPtr(Buff) then Inc(Size, StrLen(Buff)+Length(Delim));
  P := GetClearStr(Size+1);
  if NilPtr(Buff) then StrPCopy(P, S)
  else
  begin
    StrPCat(StrPCat(StrCopy(P, Buff), Delim), S);
    MyStrDispose(Buff);
  end;
  Buff := P;
end;

procedure AddPCharToBuffDelim(var Buff: PChar; AddS: PChar; Delim: PChar);
var
  P: PChar;
  Size: Cardinal;
begin
  if NilPtr(AddS) then Exit;
  if NilPtr(Delim) then Exit;
  Size := StrLen(Adds)+StrLen(Delim);
  if not NilPtr(Buff) then Inc(Size,StrLen(Buff));
  P := GetClearStr(Size+1);
  if NilPtr(Buff) then StrCopy(P, AddS)
  else
  begin
    StrCat(StrCat(StrCopy(P, Buff), Delim), AddS);
    MyStrDispose(Buff);
  end;
  Buff := P;
end;

function ConcatToPChar(S: array of String): PChar;
var
  I, Len: Cardinal;
  St: String;
begin
  Result := nil;
  Len := 0;
  for I := Low(S) to High(S) do
  begin
    St := TrimTail(S[I]);
    if St <> '' then
    begin
      Inc(Len, Length(TrimTail(S[I])));
      if I < High(S) then
        Inc(Len, 1);
    end;
  end;
  if Len = 0 then Exit;
  for I := Low(S) to High(S) do
  begin
    St := TrimTail(S[I]);
    if St <> '' then
    begin
      AddToBuff(Result, St);
      if I < High(S) then
        AddToBuff(Result, #32);
    end;
  end;
end;

function ConcatToPCharDelimEx(S: array of String; Delim: PChar; TrimBefore: Boolean): PChar;

  function GetStr(S1: String): String;
  begin
    if TrimBefore then Result := TrimTail(S1)
    else Result := S1;
  end;

var
  I, Len, LenDelim: Cardinal;
  St: String;
begin
  Result := nil;
  if not NilPtr(Delim) then LenDelim := StrLen(Delim)
  else LenDelim := 0;
  Len := 0;
  for I := Low(S) to High(S) do
  begin
    St := GetStr(S[I]);
    if St <> '' then
    begin
      Inc(Len, Length(St));
      if I < High(S) then
        Inc(Len, LenDelim);
    end;
  end;
  if Len = 0 then Exit;
  for I := Low(S) to High(S) do
  begin
    St := GetStr(S[I]);
    if St <> '' then
    begin
      if (LenDelim > 0) and (Result <> nil) then
        AddToPChar(Result, Delim);
      AddToBuff(Result, St);
    end;
  end;
end;

function ConcatToPCharDelim(S: array of String; Delim: PChar): PChar;
begin
  Result := ConcatToPCharDelimEx(S, Delim, True);
end;

function ConcatStringList(S: TStrings): PChar;
var
  I: Integer;
  Len: Cardinal;
  St: String;
begin
  Result := nil;
  if (S = nil) or (S.Count = 0) then Exit;
  Len := 0;
  for I := 0 to S.Count - 1 do
  begin
    St := TrimTail(S[I]);
    if St <> '' then
    begin
      Inc(Len, Length(TrimTail(S[I])));
      if (I < S.Count - 1) then
        if (I < S.Count - 2) then Inc(Len, 2)
        else Inc(Len, 5);
    end;
  end;
  if Len = 0 then Exit;
  for I := 0 to S.Count - 1 do
  begin
    St := TrimTail(S[I]);
    if St <> '' then
    begin
      if (I < S.Count - 1) then
        if (I < S.Count - 2) then St := Format('%s, ', [St])
        else St := Format('%s and ', [St]);
      AddToBuff(Result, St);
    end;
  end;
end;

function StrTokPos(Std, Lod: PChar): Boolean;
var
  P, Walk: PChar;
  DoFree: Boolean;
begin
  Result := True;
  Walk := Std;
  while Walk <> nil do
  begin
    DoFree := True;
    P := StrTok(Walk, #32#0);
    if NilPtr(P) then
    begin
      P := Walk;
      DoFree := False;
      Walk := nil;
    end
    else Inc(Walk);
    Result := Result and (StrPos(Lod, P) <> nil);
    if DoFree then
      MyStrDispose(P);
    if not Result then Exit;
  end;
end;

function StrCmp(CaseSensitive: Boolean; Str1, Str2: PChar): Longint;
begin
  if NilPtr(Str1) and NilPtr(Str2) then Result := 0
  else
    if NilPtr(Str1) then Result := -1
    else
      if NilPtr(Str2) then Result := 1
      else
        if CaseSensitive then
          Result := StrComp(Str1, Str2)
        else
          Result := StrIComp(Str1, Str2);
end;

function StrPosBefore(Str1, Str2, Before: PChar): PChar;
{"Before" should be found with StrPos(Str1, something). I mean is a valid Pointer in Str1}
var
  P: PChar;
begin
  if NilPtr(Before) then
    Result := StrPos(Str1, Str2)
  else
  begin
    P := StrPos(Str1, Str2);
    if P >= Before then
      Result := nil
    else
      Result := P;
  end;
end;

function InsertStr(var Buff, Str: PChar; Index: Cardinal; FreeStrAfter: Boolean): PChar;
var
  P: PChar;
  Size: Cardinal;
begin
  Result := Buff;
  if NilPtr(Buff) or NilPtr(Str) or (Index > StrLen(Buff)) then Exit;
  Size := 1 + StrLen(Buff) + StrLen(Str);
  P := GetClearStr(Size);
  StrCat(StrCat(StrLCopy(P, Buff, Index-1), Str), PChar(Buff+Index-1));
  Result := P;
  if FreeStrAfter then
    MyStrDispose(Str);
  MyStrDispose(Buff);
end;

function StrRLScan(Str: PChar; Chr: Char; MaxLen: Cardinal): PChar;
var
  Offs: Cardinal;
begin
  if MaxLen = 0 then Result := StrRScan(Str, Chr)
  else
  begin
    Offs := 0;
    Result := nil;
    while (Offs <= MaxLen) and (PChar(Str+MaxLen-Offs)^ <> Chr) do
      Inc(Offs);
    if Offs <= MaxLen then
      Result := PChar(Str+MaxLen-Offs);
  end;
end;

function HexStrToLong(Str: PChar): Longint;
const
  HexDigits: String = '123456789ABCDEF';
var
  J: Integer;
  P: PChar;
begin
  Result := 0;
  if NilPtr(Str) then Exit;
  P := Str;
  for J := StrLen(Str)-1 downto 0 do
  begin
    Inc(Result, pos(Upcase(P^), HexDigits) shl (4*J));
    Inc(P);
  end;
end;

function CreatePtrFromPChar(Str: PChar; var Size: Longint): Pointer;
var
  BuffAddr, P: PChar;
  Two: array [0..2] of Char;
  Offs: Cardinal;
  C: Char;
begin
  Result := nil;
  Size := 0;
  P := Str;
  if NilPtr(P) then Exit;
  Size := (StrLen(Str)+1) div 3;
  GetMem(Result, Size);
  BuffAddr := Result;
  FillChar(BuffAddr^, Size, #0);
  Offs := 0;
  while not NilPtr(P) do
  begin
    C := Char(HexStrToLong(StrLCopy(Two, P, 2)));
    Inc(P, 2);
    if P^ = ',' then Inc(P);
    BuffAddr[Offs] := C;
    Inc(Offs);
  end;
end;

function PtrToStr(P: Pointer; Size: Longint): String;
var
  C: PChar;
  I: Longint;
begin
  Result := '';
  if not Assigned(P) or (Size = 0) then Exit;
  C := PChar(P);
  if Size > 255 then Size := 255;
  for I := 0 to Size - 1 do
    Result := Result + Format('%.2x ', [ord(C[I])]);
end;

function IsValidIndex(P: PChar; Index: Cardinal): Boolean;
begin
  if NilPtr(P) then Result := False
  else
    Result := (Index > 0) and (Index <= StrLen(P));
end;

function DeleteANSI(var S: PChar; Index, Count: Cardinal): PChar;
var
  Len, K: Cardinal;
begin
  Result := S;
  if not IsValidIndex(S, Index) or (Count <= 0) then Exit;
  Len := StrLen(S);
  K := Len - Index + 1;
  if Count > K then
    Count := K;
  if Index + Count - 1 <> Len then
    StrMove(@S[Index-1], @S[Index+Count-1], K-Count);
  S[Len-Count] := #0;
  if StrLen(S) = 0 then
    MyStrDispose(S);
end;

function CopyANSI(S: PChar; Index, Count: Cardinal): PChar;
begin
  Result := nil;
  if not IsValidIndex(S, Index) or (Count <= 0) then Exit;
  Result := StrLDup(@S[Index-1], Count);
end;

function InsertANSI(Source: PChar; var S: PChar; Index: Cardinal): PChar;
var
  P: PChar;
begin
  Result := S;
  if NilPtr(Source) or not IsValidIndex(S, Index) then Exit;
  P := GetClearStr(StrLen(S)+StrLen(Source)+1);
  StrCat(StrCat(StrLCopy(P, S, Index-1), Source), @S[Index-1]);
  MyStrDispose(S);
  S := P;
  Result := S;
end;

function InsertANSIEx(Source, SNil: PChar; var S: PChar; Index: Cardinal): PChar;
var
  P: PChar;
begin
  Result := S;
  if (NilPtr(Source) and NilPtr(SNil)) or not IsValidIndex(S, Index) then Exit;
  if NilPtr(Source) then Source := SNil;
  P := GetClearStr(StrLen(S)+StrLen(Source)+1);
  StrCat(StrCat(StrLCopy(P, S, Index-1), Source), @S[Index-1]);
  MyStrDispose(S);
  S := P;
  Result := S;
end;

procedure ReplaceOccurances(var S: PChar; Search, Replace: PChar);
var
  Index, Tmp: PChar;
  CurPos: Cardinal;
begin
  Tmp := S;
  repeat
    Index := StrPos(Tmp, Search);
    if NilPtr(Index) then Break;
    CurPos := Index - S;
    DeleteANSI(S, CurPos+1, StrLen(Search));
    Tmp := InsertANSI(Replace, S, CurPos+1);
    Inc(Tmp, CurPos+StrLen(Replace));
  until NilPtr(Tmp);
end;

function LeadTrim(S: string; NonSpaces: Boolean): string;
var
  rLen: Integer;
  Count: Integer;
begin
  Result := S;
  rLen := Length(Result);
  if rLen = 0 then Exit;
  Count := 1;
  while (Result[Count] = #32)  or
        (NonSpaces and (Result[Count] < #32))
    do Inc(Count);
  Dec(Count);
  if Count = 0 then Exit;
  Move(Result[Count+1], Result[1], rLen-Count);
  {$IFDEF WIN32}
  SetLength(Result, rLen-Count);
  {$ELSE}
  FillChar(Result[rLen-Count+1], Count, #0);
  Dec(Result[0],Count);
  {$ENDIF}
end;

function TailTrim(S: string; NonSpaces: Boolean): string;
var
  rLen: Integer;
begin
  Result:=S;
  rLen := Length(Result);
  if rLen = 0 then Exit;
  while (rLen > 0) and
        ((Result[rLen] = #32) or
         (NonSpaces and (Result[rLen] < #32)))
    do Dec(rLen);
  {$IFDEF WIN32}
  SetLength(Result, rLen);
  {$ELSE}
  Result[0] := Char(rLen);
  {$ENDIF}
end;

function TrimLead(S: string): string;
begin
  Result := LeadTrim(S, True);
end;
function TrimTail(S: string): string;
begin
  Result := TailTrim(S, True);
end;
function TrimStr(S: string): string;
begin
  Result := TrimLead(TrimTail(S));
end;

function ReplaceString(Source, Search, RepStr: String): String;
var
  K: Integer;
begin
  Result := Source;
  K := Pos(Search, Result);
  if K = 0 then Exit;
  Delete(Result, K, Length(Search));
  if RepStr <> '' then Insert(RepStr, Result, K);
end;

{$IFNDEF WIN32}
function TrimWhite(S: String): String;
var
  I: Cardinal;
  SLen: Byte absolute S;
begin
  while (SLen > 0) and (S[SLen] <= ' ') do
    Dec(SLen);

  I := 1;
  while (I <= SLen) and (S[I] <= ' ') do
    Inc(I);
  Dec(I);
  if I > 0 then Delete(S, 1, I);

  Result := S;
end;
{$ENDIF WIN32}

function TrimLeadSpaces(S: String): String;
begin
  Result := LeadTrim(S, False);
end;
function TrimTailSpaces(S: String): String;
begin
  Result := TailTrim(S, False);
end;
function TrimSpaces(S: String): String;
begin
  Result:=TrimLeadSpaces(TrimTailSpaces(S));
end;

function pTrimLead(P: PChar): PChar;
var
  rLen, cLeft: Cardinal;
begin
  Result := P;
  if NilPtr(P) then Exit;
  rLen := StrLen(Result);
  cLeft := 0;
  while (Result[cLeft] = #32) do Inc(cLeft);
  if cLeft > 0 then
  begin
    Dec(cLeft);
    Move(Result[cLeft+1], Result[0], rLen-cLeft);
    FillChar(Result[rLen-cLeft], cLeft, #0);
  end;
end;

function pTrimTail(P: PChar): PChar;
var
  rLen, cRight: Cardinal;
begin
  Result := P;
  if NilPtr(P) then Exit;
  rLen := StrLen(Result);
  cRight := 1;
  while (cRight < rLen) and (Result[rLen-cRight] = #32) do Inc(cRight);
  Dec(cRight);
  if cRight > 0 then
    FillChar(Result[rLen-cRight], cRight, #0);
end;

function pTrimStr(P: PChar): PChar;
var
  Len, rLen, cLeft, cRight: Cardinal;
begin
  Result := P;
  if NilPtr(P) then Exit;
  rLen := StrLen(Result);
  cRight := 1;
  while (cRight <= rLen) and (Result[rLen-cRight] = #32) do Inc(cRight);
  Dec(cRight);
  cLeft := 0;
  if cRight < rLen - 1 then
    while (Result[cLeft] in [#9, #32]) do Inc(cLeft);
  if cLeft + cRight > 0 then
  begin
    Len := rLen - cLeft - cRight;
    if Len = 0 then FillChar(Result^, rLen, #0)
    else
    begin
      if cLeft > 0 then
        Move(P[cLeft], Result[0], Len);
      FillChar(Result[Len], rLen-Len, #0);
    end;
  end;
end;

function xpTrimStr(var P: PChar): PChar;
var
  rLen, cLeft, cRight: Cardinal;
begin
  Result := P;
  if NilPtr(P) then Exit;
  rLen := StrLen(Result);
  cRight := 1;
  while (cRight <= rLen) and (Result[rLen-cRight] = #32) do Inc(cRight);
  Dec(cRight);
  cLeft := 0;
  if cRight < rLen - 1 then
    while (Result[cLeft] in [#9, #32]) do Inc(cLeft);
  if cLeft + cRight > 0 then
  begin
    rLen := rLen - cLeft - cRight;
    if rLen = 0 then Result := nil
    else
    begin
      Result := GetClearStr(rLen+1);
      Move(P[cLeft], Result[0], rLen);
    end;
    MyStrDispose(P);
    P := Result;
  end;
end;

function StrRSScan(C: Char; S: String): Integer;
var
  I: Integer;
begin
  Result := -1;
  for I := Length(S) downto 1 do
    if S[I] = C then
    begin
      Result := I;
      Exit;
    end;
end;

function StringAsPChar(var S: String): PChar;
begin
  if S = '' then Result := nil
  else
  {$IFDEF WIN32}
    Result := PChar(S);
  {$ELSE}
  begin
    if Length(S) = High(S) then Dec(S[0]); { Truncate S if it's too long }
    S[Ord(Length(S))+1] := #0;             { Place null at end of string }
    Result := @S[1];                       { Return "PChar'd" string }
  end;
  {$ENDIF}
end;

function StrPCat(Dest: PChar; Source: String): PChar;
begin
  Result := Dest;
  StrCat(Dest, StringAsPChar(Source));
end;

function AddBackSlash(const S: String): String;
begin
  if (Length(S) < 255) and (S[Length(S)] <> '\') then
    Result := S + '\'
  else
    Result := S;
end;

function MultiplePos(Str: String; MatchCase: Boolean; SearchStr: array of String): Boolean;
var
  I: Integer;
begin
  Result := False;
  if Length(Str) = 0 then Exit;
  if not MatchCase then
    Str := UpperCase(TrimStr(Str));
  for I := Low(SearchStr) to High(SearchStr) do
  begin
    if not MatchCase then
      SearchStr[I] := UpperCase(SearchStr[I]);
    Result := pos(SearchStr[I], Str) > 0;
    if Result then Break;
  end;
end;

function MultipleEqual(Str: String; MatchCase: Boolean; SearchStr: array of String): Boolean;
var
  I: Integer;
begin
  Result := False;
  if Length(Str) = 0 then Exit;
  if not MatchCase then
    Str := UpperCase(TrimStr(Str));
  for I := Low(SearchStr) to High(SearchStr) do
  begin
    if not MatchCase then
      SearchStr[I] := UpperCase(SearchStr[I]);
    Result := CompareStr(Str, SearchStr[I]) = 0;
    if Result then Break;
  end;
end;

{warning: deletes searchstrs AND folowing space from str}
function MultipleFilter(Str: String; MatchCase: Boolean; SearchStr: array of String): String;
var
  I, J: Integer;
begin
  Result := '';
  Str := TrimStr(Str);
  if Length(Str) = 0 then Exit;
  if not MatchCase then Str := UpperCase(Str);
  for I := Low(SearchStr) to High(SearchStr) do
  begin
    if not MatchCase then
      SearchStr[I] := UpperCase(SearchStr[I]);
    J := pos(SearchStr[I], Str);
    if J > 0 then
      Delete(Str, J, J + Length(SearchStr[I]));
  end;
  Result := TrimStr(Str);
end;

function StripBackSlash(const S: String): String;
begin
  Result := S;
  if Result[Length(Result)] = '\' then
    SetLength(Result, Length(Result)-1);
end;

function ReturnDirectory(const APath, ADir: String): String;
var
  NewDir: String;
begin
  if pos(ADir, APath) > 0 then
  begin
    Result := APath;
    Exit;
  end;
  NewDir := APath;
  if pos('\', NewDir) < Length(NewDir) then NewDir := NewDir + '\' + ADir
  else NewDir := NewDir + ADir;
  Result := NewDir;
end;

function CompilePageSelection(Line: PChar; MinValue, MaxValue: Integer): Boolean;
var
  P, Tmp: PChar;
  C: Char;
  N, Code, Len: Integer;
begin
  Result := True;
  P := Line;
  if not NilPtr(P) then
    while not (P^ in ['0'..'9']) do
    begin
      Inc(P);
      Result := False;
    end;
  while not NilPtr(P) do
  begin
    Tmp := P;
    while P^ in ['0'..'9'] do Inc(P);
    Len := P - Tmp;
    if Len > 0 then
    begin
      C := P^;
      P^ := #0;
      val(Tmp, N, Code);
      Result := (Code = 0) and (N in [MinValue..MaxValue]);
      P^ := C;
      if P <> StrEnd(Line) then
        Inc(P);
    end;
  end;
end;

function GetPageFromPChar(var Line: PChar; var IsRange: Boolean): Integer;
var
  P, Tmp: PChar;
  C: Char;
  Code, Len: Integer;
begin
  Result := -1;
  P := Line;
  if NilPtr(P) then Exit;
  if not NilPtr(P) then
    while not (P^ in ['0'..'9']) do
      Inc(P);
  if not NilPtr(P) then
  begin
    Tmp := P;
    while P^ in ['0'..'9'] do Inc(P);
    Len := P - Tmp;
    if Len > 0 then
    begin
      C := P^;
      P^ := #0;
      val(Tmp, Result, Code);
      if (Code <> 0) then Result := -1;
      IsRange := C = '-';
      P^ := C;
      if P <> StrEnd(Line) then
        Inc(P);
      Line := P;
    end;
  end;
end;

{it will change a buffer like
   1,2,5-8,10
into
   1,2,5,6,7,8,10
}
function RecreatePagesFromPChar(var Line: PChar): PChar;
var
  P, NewSel: PChar;
  LastPage, APage: Longint;
  IsRange: Boolean;
begin
  NewSel := nil;
  P := Line;
  while not NilPtr(P) do
  begin
    APage := GetPageFromPChar(P, IsRange);
    if APage > 0 then
    begin
      if (APage > 0) then
        if not IsRange then
          AddToBuff(NewSel, Format('%d,', [APage]))
        else
        begin
          LastPage := GetPageFromPChar(P, IsRange);
          while APage <= LastPage do
          begin
            AddToBuff(NewSel, Format('%d,', [APage]));
            Inc(APage);
          end;
        end;
    end;
  end;
  P := Line;
  Line := NewSel;
  Result := Line;
  MyStrDispose(P);
end;

function CountPagesFromPChar(Line: PChar): Integer;
var
  Walk, Buff: PChar;
  IsRange: Boolean;
begin
  Result := 0;
  Buff := StrDup(Line);
  try
    RecreatePagesFromPChar(Buff);
    Walk := Buff;
    repeat
      if GetPageFromPChar(Walk, IsRange) > 0 then
        Inc(Result);
    until NilPtr(Walk);
  finally
    MyStrDispose(Buff);
  end
end;

function Filter(S: String; CS: CharSet): String;
var
  I, Len: Cardinal;
begin
  Len := 0;
  for I := 1 to Length(S) do
    if not(S[I] in CS) then
    begin
      Inc(Len);
      Result[Len] := S[I];
    end;
  SetLength(Result, Len);
end;

function Str2Word(S: String; var I: Word): Boolean;
{-Convert a string to a word, returning true if successful}
var
  Code: Integer;
begin
  Val(TrimStr(S), I, Code);
  Result := Code = 0;
  if not Result then I := Code;
end;

function WordCount(S : string; WordDelims : CharSet) : Byte;
var
  Count, I, SLen: Cardinal;
begin
  Count := 0;
  I := 1;
  sLen := Length(S);
  while I <= SLen do
  begin
    while (I <= SLen) and (S[I] in WordDelims) do
      Inc(I);
    if I <= SLen then Inc(Count);
    while (I <= SLen) and not(S[I] in WordDelims) do
      Inc(I);
  end;
  Result := Count;
end;

function WordPosition(N : Cardinal; S : string; WordDelims: CharSet) : Byte;
var
  Count, I, SLen: Cardinal;
begin
  Count  := 0;
  I      := 1;
  Result := 0;
  SLen := Length(S);
  while (I <= SLen) and (Count <> N) do
  begin
    while (I <= SLen) and (S[I] in WordDelims) do Inc(I);
    if I <= SLen then Inc(Count);
    if Count = N then Result := I
    else
      while (I <= SLen) and not(S[I] in WordDelims) do Inc(I);
  end;
end;

function CheckTopOfBuf(var P: PChar; Buffer: PChar; Offset: Cardinal): Cardinal;
begin
  P := Buffer;
  Inc(P, Offset);
  while P^ in [#10, #13, #32] do Inc(P);
  Result := P - Buffer;
end;

procedure InitialzeY2K;
var
  Year, Month, Day: Word;
begin
  DecodeDate(SysUtils.Date, Year, Month, Day);
  Y2K_Delta := 100 * (Year div 100);
end;

function HP_Selected(Str: String): Boolean;
begin
  if StrRSScan('\', Str) > 0 then
    Delete(Str, 1, StrRSScan('\', Str));
  Result := CompareText('HP LASERJET IIP', Copy(Str, 1, 15)) = 0;
end;

initialization
  InitialzeY2K;
end. { END UNIT }

The information provided above is provided "AS IS" without warranty of any kind.
Lucian Radulescu disclaims all warranties, either express or implied, including
the warranties of merchantability and fitness for a particular purpose.

In no event shall Lucian Radulescu or his suppliers be liable for any damages
whatsoever including direct, indirect, incidental, consequential, loss of business
profits or special damages, even if Lucian Radulescu or his suppliers have been
advised of the possibility of such damages.

Some states do not allow the exclusion or limitation of liability for consequential
or incidental damages so the foregoing limitation may not apply.

