unit Scale;

interface

uses
  WinProcs, Messages, Controls, Forms, StdCtrls;

const
  ScreenWidth  = 1024.0;
  ScreenHeight =  768.0;

type
  TZoomFactor = (tzSmall, tzNormal, tzLarge);

procedure ScaleForm(F: TForm);
procedure ZoomInForm(F: TForm);
procedure ZoomOutForm(F: TForm);

implementation

type
  TScaleControl = class(TControl);

procedure ScaleControl(C: TWinControl; RatioX, RatioY: Real);
var
  NewLeft, NewTop, NewHeight, NewWidth: Longint;
  I: Integer;
begin
  with C do
    for I := ControlCount - 1 downto 0 do
    begin
      NewLeft   := Round(Controls[I].Left * RatioX);
      NewTop    := Round(Controls[I].Top  * RatioY);
      NewHeight := Round(Controls[I].Height * RatioX);
      NewWidth  := Round(Controls[I].Width  * RatioY);
      Controls[I].SetBounds(NewLeft, NewTop, NewWidth, NewHeight);
      if (Controls[I] is TWinControl) then
        ScaleControl(TWinControl(Controls[I]), RatioX, RatioY)
      else
        if Controls[I].InheritsFrom(TCustomLabel) then
          TScaleControl(Controls[I]).Font.Size :=
            Trunc(RatioY * TScaleControl(Controls[I]).Font.Size);
    end;
  with TScaleControl(C), Font do
    if not ParentFont then
      Size := Trunc(RatioY * Size)
    else
      Size := TScaleControl(C.Parent).Font.Size;
end;

procedure ScaleForm(F: TForm);
var
  NewHeight, NewWidth: Longint;
  RatioX, RatioY: Real;
begin
  F.Scaled := True;
  if Screen.Width <> ScreenWidth then
  begin
    RatioX := Screen.Width / ScreenWidth;
    RatioY := Screen.Height / ScreenHeight;
    NewHeight := Round(F.Height * RatioX);
    NewWidth  := Round(F.Width  * RatioY);

    {NewHeight := Longint(F.Height) * Longint(Screen.Height) div ScreenHeight;
    NewWidth  := Longint(F.Width)  * Longint(Screen.Width)  div ScreenWidth;
    Ratio := NewWidth / F.Width;}
    F.SetBounds(F.Left, F.Top, NewWidth, NewHeight);
    ScaleControl(F, RatioX, RatioY);
  end;
end;

procedure ZoomInForm(F: TForm);
var
  NewHeight, NewWidth: Longint;
  Ratio: Real;
begin
  NewHeight := Longint(F.Height) * 5 div 4;
  NewWidth  := Longint(F.Width)  * 5 div 4;
  Ratio := NewWidth / F.Width;
  {F.SetBounds(F.Left, F.Top, NewWidth, NewHeight);}
  ScaleControl(F, Ratio, Ratio);
end;

procedure ZoomOutForm(F: TForm);
var
  NewHeight, NewWidth: Longint;
  Ratio: Real;
begin
  NewHeight := Longint(F.Height) * 4 div 5;
  NewWidth  := Longint(F.Width)  * 4 div 5;
  Ratio := NewWidth / F.Width;
  {F.SetBounds(F.Left, F.Top, NewWidth, NewHeight);}
  ScaleControl(F, Ratio, Ratio);
end;

end.

