{*******************************************************}
{                                                       }
{       Get/Set/Delete Volume label                     }
{       ---------------------------                     }
{       16 bit only                                     }
{       Copyright (C), Borland International Inc.       }
{                                                       }
{       You can get this from                           }
{         http://www.borland.com/devsupport/delphi/ti_list/TI2854.html }
{                                                       }
{*******************************************************}

unit VolLabel;

{$I ALLLIB.DEF}

interface

uses
  WinProcs, SysUtils;

type

  EInterruptError = class(Exception);
  EDPMIError      = class(EInterruptError);
  EDosAllocError  = class(EInterruptError);

  Str11           = String[11];

{$IFNDEF WIN32}
procedure SetVolumeLabel(NewLabel: Str11; Drive: Char);
function  GetVolumeLabel(Drive: Char): Str11;
procedure DeleteVolumeLabel(Drv: Char);
function ReadIt(Drive: Byte; var Buffer): Boolean;
function SerialNumber(Drive: Byte): String;
{$ENDIF}

{function  AbsoluteDiskRead(Drive: Byte; Buffer: PChar): Boolean;}

implementation

type

  PRealModeRegs = ^TRealModeRegs;
  TRealModeRegs = record
    case Integer of
      0: (EDI, ESI, EBP, EXX, EBX, EDX, ECX, EAX: Longint;
	  Flags, ES, DS, FS, GS, IP, CS, SP, SS: Word);
      1: (DI, DIH, SI, SIH, BP, BPH, XX, XXH: Word;
	  case Integer of
	    0: (BX, BXH, DX, DXH, CX, CXH, AX, AXH: Word);
	    1: (BL, BH, BLH, BHH, DL, DH, DLH, DHH,
		CL, CH, CLH, CHH, AL, AH, ALH, AHH: Byte));
  end;

{ RealModeInt invokes int 31h function 0300h to simulate }
{ a real mode interrupt from protected mode.             }
procedure RealModeInt(Int: Byte; var Regs: TRealModeRegs);
var
  ErrorFlag: Boolean;
begin
  asm
    MOV ErrorFlag, 0       { assume success }
    MOV AX, 0300h          { function 300H }
    MOV BL, Int            { real mode interrupt to execute }
    MOV BH, 0              { required }
    MOV CX, 0              { stack words to copy, assume zero }
    LES DI, Regs           { ES:DI = Regs }
    INT 31H                { DPMI int 31h }
    JNC @@End              { carry flag set on error }
  @@Error:
    MOV ErrorFlag, 1       { return false on error }
  @@End:
  end;
  if ErrorFlag then
    raise EDPMIError.Create('Failed to execute DPMI interrupt.');
end;

{$IFNDEF WIN32}
type

  PExtendedFCB = ^TExtendedFCB;
  TExtendedFCB = Record
    ExtendedFCBflag : Byte;
    Reserved1       : array[1..5] of Byte;
    Attr            : Byte;
    DriveID         : Byte;
    FileName        : array[1..8] of Char;
    FileExt         : array[1..3] of Char;
    CurrentBlockNum : Word;
    RecordSize      : Word;
    FileSize        : LongInt;
    PackedDate      : Word;
    PackedTime      : Word;
    Reserved2       : array[1..8] of Byte;
    CurrentRecNum   : Byte;
    RandomRecNum    : LongInt;
  end;

{ DriveLetterToNumber converts a character drive letter into its numerical equiv. }
function DriveLetterToNumber(Drive: Char): Byte;
begin
  if Drive in ['a'..'z'] then
    Drive := Chr(Ord(Drive) - 32);
  if not (Drive in ['A'..'Z']) then
    raise EConvertError.CreateFmt('Cannot convert %s to drive number.', [Drive]);
  Result := Ord(Drive) - 64;
end;

{ PadVolumeLabel pads Volume Label string with spaces. }
procedure PadVolumeLabel(var Name: Str11);
begin
  FillChar(Name[ord(Succ(Name[0]))], 11 - ord(Name[0]), #32);
  Name[0] := Chr(11);
end;

{ GetVolumeLabel returns volume label of a disk. }
function GetVolumeLabel(Drive: Char): Str11;
var
  SR: TSearchRec;
  SearchString: String[7];
  P: Byte;
begin
  SearchString := Drive + ':\*.*';
  if FindFirst(SearchString, faVolumeID, SR) = 0 then
  begin
    P := Pos('.', SR.Name);
    if P > 0 then                            { if it has a dot... }
    begin
      Result := '           ';               { pad spaces between name }
      Move(SR.Name[1], Result[1], P - 1);    { and extension }
      Move(SR.Name[P + 1], Result[9], 3);
    end
    else
    begin                                    { otherwise, pad to end }
      Result := SR.Name;
      PadVolumeLabel(Result);
    end;
  end
  else
    Result := '';
  FindClose(SR);
end;

procedure DeleteVolumeLabel(Drv: Char);
{ procedure deletes volume label from given drive }
var
  CurName: Str11;
  FCB: TExtendedFCB;
  ErrorFlag: WordBool;
begin
  ErrorFlag := False;
  CurName := GetVolumeLabel(Drv);        { get current volume label }
  FillChar(FCB, SizeOf(FCB), 0);         { initialize FCB with zeros }
  with FCB do begin
    ExtendedFCBflag := $FF;              { always }
    Attr := faVolumeID;                  { Volume ID attribute }
    DriveID := DriveLetterToNumber(Drv); { Drive number }
    Move(CurName[1], FileName, 8);       { must enter volume label }
    Move(CurName[9], FileExt, 3);
  end;
  asm
    PUSH DS                              { preserve ds }
    MOV AX, SS                           { put seg of FCB (ss) in ds }
    MOV DS, AX
    LEA DX, FCB                          { put offset of FCB in dx }
    MOV AX, 1300H                        { function 13h }
    Call DOS3Call                        { invoke int 21h }
    POP DS                               { restore ds }
    CMP AL, 00H                          { check for success }
    JE @@End
  @@Error:                               { set flag on error }
    MOV ErrorFlag, 1
  @@End:
  end;
  if ErrorFlag then
    raise EInterruptError.Create('Failed to delete volume name.');
end;

procedure SetVolumeLabel(NewLabel: Str11; Drive: Char);
{ procedure sets volume label of a disk.  Note that this procedure }
{ deletes the current label before setting the new one.  This is }
{ required for the set function to work. }
var
  Regs: TRealModeRegs;
  FCB: PExtendedFCB;
  Buf: Longint;
  Selector: Word;
begin
  PadVolumeLabel(NewLabel);
  if GetVolumeLabel(Drive) <> '' then           { if it has label...   }
    DeleteVolumeLabel(Drive);                   { delete the label     }
  Buf := GlobalDOSAlloc(SizeOf(PExtendedFCB));  { allocate real buffer }
  if Buf = 0 then                               { failed to allocate   }
    raise EDosAllocError.Create('SetVolumeLabel failed to allocate DOS memory.');
  Selector := LoWord(Buf);                      { get selector         }
  FCB := Ptr(Selector, 0);
  FillChar(FCB^, SizeOf(FCB), 0);               { init FCB with zeros  }
  with FCB^ do begin
    ExtendedFCBflag := $FF;                     { required             }
    Attr := faVolumeID;                         { Volume ID attribute  }
    DriveID := DriveLetterToNumber(Drive);      { Drive number         }
    Move(NewLabel[1], FileName, 8);             { set new label        }
    Move(NewLabel[9], FileExt, 3);
  end;
  FillChar(Regs, SizeOf(Regs), 0);              { Fill FCB with nul    }
  with Regs do begin
    DS := HiWord(Buf);                          { SEGMENT of FCB       }
    DX := 0;                                    { offset = zero        }
    AX := $1600;                                { function 16H         }
  end;
  try
    RealModeInt($21, Regs);                     { create file          }
    if (Regs.AL <> 0) then                      { check for success    }
      raise EInterruptError.Create('Failed to create volume label.');
  finally
    GlobalDosFree(Selector); {this is not in original code, dunno why!}
  end;
end;

function ReadIt(Drive: Byte; var Buffer): Boolean;
var
  Buff: array [0..511] of Char;
begin
asm
	MOV	AL,Drive
	MOV	DX,0
	MOV	CX,1
	LEA	BX,Buff
	INT	25H
	POP	DX
	JNC	@@1
	MOV	AX,0
	JMP	@@2
@@1:	MOV	AX,1
@@2:
end;
  if Result then
    Move(Buff, Buffer, 512);
end;

function SerialNumber(Drive: Byte): String;
var
  Buff: array [0..511] of Char;
  Serial: Longint;
begin
  FillChar(Buff, SizeOf(Buff), #0);
  ReadIt(Drive, Buff);
  Move(Buff[39], Serial, 4);
  Result := Format('%.4x-%.4x', [HiWord(Serial), LoWord(Serial)]);
end;
{$ENDIF}

{function AbsoluteDiskRead(Drive: Byte; Buffer: PChar): Boolean;
var
  Regs: TRealModeRegs;
begin
  with Regs do
  begin
    AL := Drive;
    DX := 0;
    CX := 1;
    BX := LoWord(Longint(Buffer));
    DS := HiWord(Longint(Buffer));
  end;
  RealModeInt($25, Regs);
  Result := True;
end;
}
end.

The information provided above is provided "AS IS" without warranty of any kind.
Lucian Radulescu disclaims all warranties, either express or implied, including
the warranties of merchantability and fitness for a particular purpose.

In no event shall Lucian Radulescu or his suppliers be liable for any damages
whatsoever including direct, indirect, incidental, consequential, loss of business
profits or special damages, even if Lucian Radulescu or his suppliers have been
advised of the possibility of such damages.

Some states do not allow the exclusion or limitation of liability for consequential
or incidental damages so the foregoing limitation may not apply.

