{*******************************************************}
{                                                       }
{       Some API stuff - Implementation Part            }
{       ------------------------------------            }
{       ADV Compiled                                    }
{       Copyright (C), Lucian Radulescu                 }
{       E-mail:    ljee@interlog.com                    }
{       Home Page: http://www.interlog.com/~ljee        }
{                                                       }
{*******************************************************}

unit WinTools;

{$I WINTOOLS.INT}

implementation

{$IFDEF WIN32}
  {$IFDEF SAFEVOLLABEL}
    {$UNDEF SAFEVOLLABEL}
  {$ENDIF}
{$ENDIF}

uses
  {$IFDEF SAFEVOLLABEL} VolLabel, {$ENDIF}
  PCharLIB;

function GetPtr(MemPtr: Pointer; Index: LongInt): Pointer;
var
  {$IFNDEF WIN32}
  Selector, OffSet: Word;
  {$ENDIF}
  MemHandle: THandle;
begin
  {$IFDEF WIN32}
  MemHandle := GlobalHandle(MemPtr);
  {Result := Pointer(LongInt(MemPtr) + Index);}
  Result := Pointer(MemHandle + Index);
  {$ELSE}
  MemHandle := GlobalHandle(PtrRec(MemPtr).Seg);
  Selector := (Index div 65536) * 8 + MemHandle;
  Offset := Index mod 65536;
  Result := Ptr(Selector, Offset);
  {$ENDIF}
end;

function  Check_Available_Resources(Option: Word; RaiseErr: Boolean): Boolean;
{$IFDEF WIN32}
var
  PMem: TMemoryStatus;
{$ENDIF}
begin
  {$IFDEF WIN32}
  PMem.dwLength := SizeOf(TMemoryStatus);
  GlobalMemoryStatus(PMem);
  Result := PMem.dwMemoryLoad < 80;
  {$ELSE}
  Result := True;
  if Option and ckSYS = ckSYS then
    Result := Result and (GetFreeSystemResources(GFSR_SYSTEMRESOURCES) > 15);
  if Option and ckUSR = ckUSR then
    Result := Result and (GetFreeSystemResources(GFSR_USERRESOURCES) > 15);
  if Option and ckGDI = ckGDI then
    Result := Result and (GetFreeSystemResources(GFSR_GDIRESOURCES) > 15);
  {$ENDIF}
  if not Result and RaiseErr then
    Raise ENotEnoughMemory.Create(
     'Not enough memory to complete this operation !'#13#10+
     'Please close other applications and try again.');
end;

function GetDropppedFiles(hDrop: THandle): TStringList;
var
  Pt: TPoint;
  LenName, NoFiles: Integer;
  FileName: array [0..MAX_NAME_SIZE] of Char; {can change this later, if not enough}
  I: UINT;
begin
  Result := nil;
  if not DragQueryPoint(hDrop, Pt) then Exit;
  NoFiles := DragQueryFile(hDrop, {$IFDEF WIN32}$FFFFFFFF{$ELSE}Word(-1){$ENDIF}, nil, 0);
  Result := TStringList.Create;
  if NoFiles > 0 then
    for I := 0 to NoFiles - 1 do
    begin
      LenName := DragQueryFile(hDrop, I, nil, 0);
      if LenName > MAX_NAME_SIZE then Continue {avoid a memory violation}
      else Inc(LenName);
      DragQueryFile(hDrop, I, EmptyPChar(FileName, MAX_NAME_SIZE+1), LenName);
      Result.Add(LowerCase(StrPas(FileName)));
    end;
  DragFinish(hDrop);
  if Result.Count = 0 then
  begin
    Result.Free;
    Result := nil;
  end;
end;

function GetTemporaryFileName(Prefix, NewExt: PChar): String;
var
  {$IFDEF WIN32}
  Path: array [0..MAX_NAME_SIZE] of char;
  {$ELSE}
  Path: Char;
  {$ENDIF}
  NameBuff: array [0..MAX_NAME_SIZE] of char;
begin
  {$IFDEF WIN32}
  GetTempPath(MAX_NAME_SIZE, EmptyPChar(Path, MAX_NAME_SIZE+1));
  {$ELSE}
  Path := GetTempDrive('c');
  {$ENDIF}
  if NilPtr(Prefix) then
    GetTempFileName(Path, 'jee', 0, EmptyPChar(NameBuff, MAX_NAME_SIZE+1))
  else
    GetTempFileName(Path, Prefix, 0, EmptyPChar(NameBuff, MAX_NAME_SIZE+1));
  Result := StrPas(NameBuff);
  if not NilPtr(NewExt) then
  begin
    RenameFile(Result, ChangeFileExt(Result, StrPas(NewExt)));
    Result := ChangeFileExt(Result, StrPas(NewExt));
  end;
  Result := Result + #0;
  SysUtils.DeleteFile(Result);
end;

function Application_ExeName: string;
var
  FileName: array [0..MAX_NAME_SIZE] of Char;
{$IFDEF WIN32}
begin
  if GetModuleFileName(0, EmptyPChar(FileName, MAX_NAME_SIZE+1), MAX_NAME_SIZE) > 0 then
    Result := StrPas(FileName)
  else
    Result := '';
end;
{$ELSE}
  Task: TTaskEntry;
begin
  Result := '';
  if GetCurrentTask <> 0 then
  begin
    Fillchar(Task, SizeOf(Task), #0);
    Task.dwSize := SizeOf(Task);
    if not TaskFindHandle(@Task, GetCurrentTask) then Exit;
    if GetModuleFileName(Task.hModule, EmptyPChar(FileName, MAX_NAME_SIZE+1), MAX_NAME_SIZE) > 0 then
      if FileName[0] <> #0 then
	Result := StrPas(FileName);
  end;
end;
{$ENDIF}

function GetEnvVar(EnvVar: PChar): String;
var
  {$IFNDEF WIN32}P: PChar;{$ENDIF}
  S: array [0..MAX_NAME_SIZE] of Char;
begin
  Result := '';                                { return empty string on fail }
  {$IFDEF WIN32}
  GetEnvironmentVariable(EnvVar,EmptyPChar(S, MAX_NAME_SIZE+1),255);{ retrieve }
  Result := StrPas(S);                         { return the string }
  {$ELSE}
  P := GetDOSEnvironment;                      { retrieve pointer to env vars }
  StrCat(StrCopy(S, EnvVar), '=');             { append "=" sign to string }
  while P^ <> #0 do
    { does first environment variable match EnvVar? }
    if StrLIComp(P, EnvVar, StrLen(EnvVar)) <> 0 then
      Inc(P, StrLen(P) + 1)                    { if not, then go to next }
    else
    begin
      Inc(P, StrLen(EnvVar)+1);                { if so, the get value, skip '=' }
      Result := StrPas(S);                     { return the string }
      Break;                                   { get out of loop }
    end;
  {$ENDIF}
end;

function JustOnePeek: Boolean;
var
  AppMsg: TMsg;
begin
  Result := PeekMessage(AppMsg, 0, 0, 0, pm_Remove);
  if Result then
  begin
    TranslateMessage(AppMsg);
    DispatchMessage(AppMsg);
  end;
end;

procedure ProcessMsgAsIdle;
begin
  while JustOnePeek do;
end;

procedure ProcessWaitMsgAsIdle;
begin
  WaitMessage;
  ProcessMsgAsIdle;
end;

procedure WaitIdle(DelayTime: LongInt);{milisecs}
var
  StartTime: LongInt;
begin
  if (DelayTime = 0) then Exit;
  StartTime := GetTickCount;
  repeat JustOnePeek;
  until GetTickCount - StartTime >= DelayTime;
end;

function WinExecAndWaitEx(Path: PChar; Visibility: Word; gTCHandle: THandle): Word;
{$IFDEF WIN32}
var Code: DWORD;
{$ENDIF}
begin
  Result := WinExec(Path, Visibility);
  if Result <= hInstance_Error then Exit
  else
  begin
    {$IFDEF WIN32}
    while GetExitCodeProcess(Result, Code) do
      if Code <> STILL_ACTIVE then Break
      else
      begin
        JustOnePeek;
        WaitMessage;
      end;
    Result := GetLastError;
    {$ELSE}
    while GetModuleUsage(Result) > 0 do
    begin
      JustOnePeek;
      WaitMessage;
    end;
    Result := hInstance_Error;
    {$ENDIF}
    if not IsWindowVisible(gTCHandle) then
      ShowWindow(gTCHandle, SW_SHOW);
  end;
end;

function IdleNow(Handle: THandle): THandle;
{$IFDEF WIN32}
var Code: DWORD;
{$ENDIF}
begin
  {$IFDEF WIN32}
  while GetExitCodeProcess(Handle, Code) do
    if Code <> STILL_ACTIVE then Break
    else ProcessWaitMsgAsIdle;
  Result := GetLastError;
  {$ELSE}
  if Handle <= hInstance_Error then Exit;
  while GetModuleUsage(Handle) > 0 do
    ProcessWaitMsgAsIdle;
  Result := hInstance_Error;
  {$ENDIF}
end;

function DoShellExecute(Handle: HWND; FileToOpen, CmdLine, Directory: PChar; Visibility: Integer): THandle;
{$IFDEF WIN32}
var
  lpExecInfo: TSHELLEXECUTEINFOA;
{$ENDIF}
begin
  {$IFDEF WIN32}
  with lpExecInfo do
  begin
    cbSize := SizeOf(lpExecInfo);
    fMask := SEE_MASK_DOENVSUBST or SEE_MASK_NOCLOSEPROCESS;
    wnd := Handle;
    lpVerb := nil;
    lpFile := FileToOpen;
    lpParameters := CmdLine; {command line}
    lpDirectory := Directory;{use current directory}
    nShow := Visibility;
  end;
  ShellExecuteEx(@lpExecInfo);
  Result := lpExecInfo.hProcess;
  {$ELSE}
  Result := ShellExecute(Handle, 'Open', FileToOpen, CmdLine, Directory, Visibility);
  {$ENDIF}
end;

function ShellWait(Wnd: HWND; FileToOpen, CmdLine, Directory: PChar; Visibility: Word): Cardinal;
begin
  Result := DoShellExecute(Wnd, FileToOpen, CmdLine, Directory, Visibility);
  {$IFDEF WIN32}
  if GetLastError = 0 then
    IdleNow(Result);
  {$ELSE}
  if Result <= hInstance_Error then Exit
  else
    IdleNow(Result);
  {$ENDIF}
end;

function AllocDataSpace(var Data: Pointer; Size: Longint; UseGlobal: Boolean): Longint;
begin
  if UseGlobal then Data := GlobalAllocPtr(GHND, Size)
  else
  try
    GetMem(Data, Size);
    if Assigned(Data) then
      FillChar(Data^, Size, #0);
  except
    on E: EOutOfMemory do { our app doesn't jump in ... hell }
      Data := nil;        { you have to check for nil data   }
  end;
  if not Assigned(Data) then Result := 0
  else Result := Size;
end;

procedure FreeData(var Data: Pointer; Size: Longint; GlobalUsed, ForcedGlobalFree: Boolean);
begin
  if not Assigned(Data) then Exit;
  if GlobalUsed or ForcedGlobalFree then GlobalFreePtr(Data)
  else FreeMem(Data, Size);
  Data := nil;
end;

function HugeRead(const FileName: String; Mode: Integer): THandle;
var
  Size: LongInt;
  FileHandle: Integer;
  P: Pointer;
begin                                                       { begin body               }
  Result:=0;                                                { prepare for worse        }
  FileHandle:=FileOpen(FileName, Mode);                     { try to open file         }
  if FileHandle < 0 then Exit;                              { exit on error            }
  try
    Size:=FileSeek(FileHandle,0,2);                         { get file size            }
    Result:=GlobalAlloc(GHND, Size);                        { alloc size               }
    if Result <> 0 then                                     { succes, go on            }
    begin                                                   { bb                       }
      FileSeek(FileHandle,0,0);                             { positioning to begining  }
      P:=GlobalLock(Result);                                { lock memory, get pointer }
      if P <> nil then
      begin                                                 { succes                   }
	FileRead(FileHandle, P^, Size);                     { read file into buffer P^ }
	GlobalUnlock(Result);
      end
      else {some error}                                     { failed                   }
      begin                                                 { bb                       }
	GlobalFree(Result);                                 { free allocated memory    }
	Result:=0;                                          { return worse             }
      end;                                                  { eb                       }
    end;                                                    { eb                       }
  finally
    FileClose(FileHandle);                                  { close file               }
  end;                                                      { end body                 }
end;                                                        { end body                 }

function HugeReadPtr(FileName: PChar; var Size: Longint): Pointer;
var
  FileHandle: Integer;
begin                                                       { begin body               }
  Result:=nil;                                              { prepare for worse        }
  Size := 0;
  FileHandle:=FileOpen(StrPas(FileName), fmOpenRead);       { try to open file         }
  if FileHandle < 0 then Exit;                              { exit on error            }
  try
    Size:=FileSeek(FileHandle,0,2);                         { get file size            }
    if Size > 0 then                                        { only if not empty file   }
    begin
      FileSeek(FileHandle,0,0);                             { positioning to begining  }
      AllocDataSpace(Result,Size,True);                     { alloc size               }
      if Assigned(Result) then                              { succes, go on            }
        FileRead(FileHandle, Result^, Size);                { read file into buffer P^ }
    end;
  finally
    FileClose(FileHandle);                                  { close file               }
  end;                                                      { end body                 }
end;                                                        { end body                 }

function HugePut(const FileName: String; Mode: Integer; P: Pointer): LongInt;
var
  BytesRead, Size: LongInt;
  FileHandle: Integer;
begin
  Result:=0;
  if not Assigned(P) then Exit;
  FileHandle:=FileOpen(FileName, Mode);                     { try to open file         }
  if FileHandle < 0 then Exit;                              { exit on error            }
  try
    Size:=FileSeek(FileHandle,0,2);                         { get file size            }
    FileSeek(FileHandle,0,0)  ;                             { positioning to begining  }
    BytesRead:=FileRead(FileHandle, P^, Size);              { read file into buffer P^ }
    Result:=BytesRead;                                      { should be equal to size  }
  finally
    FileClose(FileHandle);                                  { close file               }
  end;
end;

function HugeWrite(const FileName: String; P: Pointer; Size: LongInt): Boolean;
var
  FileHandle: Integer;
  BytesWrite: LongInt;
begin
  Result := False;
  if FileExists(FileName) or (P = nil) then Exit;           { test some initial conds  }
  FileHandle:=FileCreate(FileName);                         { try to open file         }
  if FileHandle < 0 then Exit;                              { exit on error            }
  try
    BytesWrite:=FileWrite(FileHandle, P^, Size);            { read file into buffer P^ }
    Result:=BytesWrite=Size;
  finally
    FileClose(FileHandle);                                  { close file               }
  end;
end;

function HugeWriteAdd(const FileName: String; P: Pointer; Size: LongInt): Boolean;
var
  FileHandle: Integer;
begin
  if not FileExists(FileName) then
    Result := HugeWrite(FileName, P, Size)
  else
  begin
    Result := False;
    FileHandle:=FileOpen(FileName, fmOpenReadWrite);        { try to open file         }
    if FileHandle < 0 then Exit;                            { exit on error            }
    try
      FileSeek(FileHandle, 0, 2);                           { position at the end      }
      if P <> nil then                                      { succes                   }
	FileWrite(FileHandle, P^, Size);                    { write file from buffer P^}
      Result := True;
    finally
      FileClose(FileHandle);                                { close file               }
    end;
  end;
end;

function GetResBuffer(ResName: PChar): PChar;
var
  ResIdHandle, ResDataHandle: THandle;
  ResDataPtr: Pointer;
  Size: Cardinal;
begin
  Result := nil;
  ResIDHandle := FindResource(HInstance, ResName, MakeIntResource(10));
  if ResIDHandle = 0 then Exit;
  ResDataHandle := LoadResource(HInstance, ResIDHandle);
  if ResDataHandle = 0 then Exit;
  {$IFDEF WIN32}
  ResDataPtr := Pointer(ResDataHandle);
  Size := StrLen(ResDataPtr)+1;
  {$ELSE}
  Size := GlobalSize(ResDataHandle);
  ResDataPtr := LockResource(ResDataHandle);
  {$ENDIF WIN32}
  try
    if (ResDataPtr <> nil) then
      Result := StrLCopy(GetClearStr(Size+1), PChar(ResDataPtr), Size);
  finally
    {$IFNDEF WIN32}
    UnlockResource(ResDataHandle);
    FreeResource(ResDataHandle);
    {$ENDIF WIN32}
  end;
end;

function WaitAndClose_TTY(Wnd: HWND; Caption: PChar): Boolean;
var
  {$IFDEF WIN32}
  Code: DWORD;
  {$ENDIF}
  XWnd: HWND;
begin
  Result := False;
  if Wnd <= hInstance_Error then Exit;
  {$IFDEF WIN32}
  while GetExitCodeProcess(Wnd, Code) do
  if Code <> STILL_ACTIVE then Break
  else
  {$ELSE}
  while GetModuleUsage(Wnd) > 0 do
  {$ENDIF}
  begin
    ProcessMsgAsIdle;
    XWnd := FindWindow('tty', Caption);
    if XWnd > 0 then
      SendMessage(XWnd, WM_CLOSE, 0, 0);
  end;
  Result := XWnd > 0;
end;

procedure QuickFormatDrive(AOwner: TComponent; QuickFormat: Boolean;
  Drive: Char; DiskLabel: String);
const
  ShellTextFileName: String = 'SHELL.TXT';
  ShellBatchFileName: String = 'SHELL.BAT'#0;
  SArr: array [Boolean] of String = ('Formatting disk in drive ', 'Quick formatting disk in drive ');
  FStr: array [Boolean] of String = ('FORMAT %s: /U < %s', 'FORMAT %s: /U /Q < %s');
var
  F: TextFile;
begin
  AssignFile(F, ShellTextFileName);
  Rewrite(F);
  Writeln(F);
  {$IFDEF SAFEVOLLABEL}
  Writeln(F);
  {$ELSE}
  Writeln(F, DiskLabel);
  {$ENDIF}
  Writeln(F, 'N');
  Writeln(F);
  CloseFile(F);
  AssignFile(F, ShellBatchFileName);
  Rewrite(F);
  Writeln(F, Format(FStr[QuickFormat], [Drive, ShellTextFileName]));
  CloseFile(F);
  try
    WaitAndClose_TTY(DoShellExecute(0, @ShellBatchFileName[1], nil, nil, SW_SHOWMINIMIZED),
      'Finished - shell');
  finally
    SysUtils.DeleteFile(ShellBatchFileName);
    SysUtils.DeleteFile(ShellTextFileName);
  end;
  {$IFDEF SAFEVOLLABEL}
  SetVolumeLabel(DiskLabel, Drive);
  {$ENDIF}
end;

procedure LabelTheDisk(AOwner: TComponent; Drive: Char; DiskLabel: String; DosLabelExe: Boolean);
{$IFNDEF SAFEVOLLABEL}
const
  LabelExeName: array [Boolean] of PChar = ('VL.EXE', 'LABEL.EXE');
  Finnished: array [Boolean] of PChar = ('Finished - Vl', 'Finished - Label');
var
  C: array [0..12] of Char;
{$ENDIF}
begin
  {$IFDEF SAFEVOLLABEL}
  SetVolumeLabel(DiskLabel, Drive);
  {$ELSE}
  WaitAndClose_TTY(DoShellExecute(0, LabelExeName[DosLabelExe],
	   StrPCopy(C, Format('%s: %s',[Drive, Copy(DiskLabel, 1, 11)])),
	   nil, SW_SHOWMINIMIZED), Finnished[DosLabelExe])
  {$ENDIF}
end;

function GetDiskLabel(AOwner: TComponent; Drive: Char): String;
{$IFNDEF SAFEVOLLABEL}
const
  NoStr: String = #13#10'N'#13#10#13#10;
  FileStr: String = '<IN.TMP >OUT.TMP';
var
  FBatch, Fin, Fout: TextFile;
  Line: String;
{$ENDIF}
begin
  {$IFDEF SAFEVOLLABEL}
  Result := GetVolumeLabel(Drive);
  {$ELSE}
  Result := '';
  AssignFile(Fin, 'IN.TMP');
  Rewrite(Fin);
  write(Fin,NoStr);
  CloseFile(Fin);

  AssignFile(FBatch, 'SHELL.BAT');
  Rewrite(FBatch);
  Writeln(FBatch, Format('LABEL %s: %s', [Drive, FileStr]));
  Writeln(FBatch);
  CloseFile(FBatch);
  try
    WaitAndClose_TTY(DoShellExecute(0, 'SHELL.BAT', nil, nil, SW_SHOWMINIMIZED),
      'Finished - Shell');
    AssignFile(Fout, 'OUT.TMP');
    Reset(Fout);
    readln(Fout, Line);
    CloseFile(Fout);
    Line := TrimStr(Line);
    Result := Copy(Line, StrRSScan(#32, Line)+1, 11);
  finally
    Erase(Fin);
    Erase(FBatch);
    Erase(Fout);
  end;
  {$ENDIF}
end;

function HugeWriteDateCheck(const FileName: String; P: Pointer; DOSDate, Size: LongInt;
  Overwrite: Boolean): Boolean;
var
  FileHandle: Integer;
  DOSFileDateFound: LongInt;
begin
  Result := False;
  if P = nil then Exit;                                     { Heeey, what's going on?  }
  if FileExists(FileName) then                              { test if file exists      }
  begin
    FileHandle := FileOpen(FileName, fmOpenRead);           { open file read only mode }
    DOSFileDateFound := FileGetDate(FileHandle);            { get dos-creation date    }
    FileClose(FileHandle);                                  { close the file           }
    if (DosFileDateFound >= DOSDate) or not Overwrite then  { don't overwrite files    }
      Exit;
  end;
  FileHandle := FileCreate(FileName);                       { try to open file         }
  if FileHandle < 0 then Exit;                              { exit on error            }
  try
    FileWrite(FileHandle, P^, Size);                        { write file from buffer P^}
    FileSetDate(FileHandle, DOSDate);                       { set file date stamp      }
    Result := True;                                         { return true              }
  finally
    FileClose(FileHandle);                                  { close the file           }
  end;
end;

function ResourceToFile(Resource: PChar; Path: String): Boolean;
var
  ResIdHandle: THandle;
  ResDataHandle: THandle;
  TF: PFileRes;
  P: Pointer;
  FileName: String;
begin
  Result := False;
  ResIDHandle := FindResource(HInstance, Resource, MakeIntResource(10));
  ResDataHandle := LoadResource(HInstance, ResIDHandle);
  TF := LockResource(ResDataHandle);
  if (ResIDHandle = 0) or (ResDataHandle = 0) or (TF = nil) then Exit;
  P := GetPtr(TF, 8);
  FileName := AddBackSlash(Path) + StrPas(Resource);
  Insert('.', FileName, Length(FileName) - 2);
  try
    Result := HugeWriteDateCheck(FileName, P, TF^.DOSDate, TF^.Size, True);
  finally
    UnlockResource(ResDataHandle);
    FreeResource(ResDataHandle);
  end;
end;

function ResourceToStrings(Resource: PChar): TStringList;
var
  ResIdHandle: THandle;
  ResDataHandle: THandle;
  TF: PFileRes;
  P, S: PChar;
begin
  Result := TStringList.Create;
  ResIDHandle := FindResource(HInstance, Resource, MakeIntResource(10));
  ResDataHandle := LoadResource(HInstance, ResIDHandle);
  TF := LockResource(ResDataHandle);
  if (ResIDHandle = 0) or (ResDataHandle = 0) or (TF = nil) then Exit;
  try
    P := GetPtr(TF, 8);
    while not NilPtr(P) do
    begin
      S := StrTok(P, #13#10);
      try
	if NilPtr(S) then
	begin
	  S := StrDup(P);
	  P := nil;
	end
	else
	  Inc(P, 2);
	Result.Add(StrPas(S)+#0);
      finally
	MyStrDispose(S);
      end;
    end;
  finally
    UnlockResource(ResDataHandle);
    FreeResource(ResDataHandle);
  end;
end;

function ExecCompress(CompressExeName, Path, Directory: PChar): Boolean;
begin
  Result := WaitAndClose_TTY(DoShellExecute(0, CompressExeName, Path, Directory, SW_SHOWMINIMIZED),
              'Finished - Compress');
end;

procedure UncompressFile(Source, Target: String);
var
  SourceHandle, TargetHandle: Integer;
  OpenBuf: TOFStruct;
begin
  Source := Source + #0;
  Target := Target + #0;
  SourceHandle := LZOpenFile(@Source[1], OpenBuf, of_Share_Deny_Write or of_Read);
  if SourceHandle = -1 then
    raise EInOutError.Create('Could not open '+Source);
  try
    TargetHandle := LZOpenFile(@Target[1], OpenBuf, of_Share_Exclusive or of_Write or of_Create);
    if TargetHandle = -1 then
      raise EInOutError.Create('Could not create '+Target);
    try
      if LZCopy(SourceHandle, TargetHandle) < 0 then
        raise EInOutError.Create('Could not copy '+Source+' to '+ Target);
    finally
      LZClose(TargetHandle);
    end;
  finally
    LZClose(SourceHandle);
  end;
end;

{added on Feb 11/1998 and is not tested yet}
procedure SwapForms(NewForm, OldForm: THandle; SwapSizes: Boolean);
var
  R: TRect;
begin
  if SwapSizes then
  begin
    GetWindowRect(OldForm, R);
    MoveWindow(NewForm, R.Left, R.Top, R.Right - R.Left, R.Bottom - R.Top, True);
  end;
  ShowWindow(NewForm, SW_SHOWNORMAL);
  ShowWindow(OldForm, SW_SHOWMINIMIZED);
end;

{ muc si sfirc !!! cool way of swapping applications }
procedure SwapApps(NewForm, OldForm: THandle; SW_WHAT: Word; SwapSizes: Boolean);
var
  R   : TRect;
  Wnd : THandle;
begin
  if SwapSizes then
    with R do begin
      GetWindowRect(OldForm, R);
      MoveWindow(NewForm, Left, Top, Right-Left, Bottom-Top, True);
    end;
  Wnd := NewForm;
  if SW_WHAT = SW_HIDE then begin
    ShowWindow(Wnd, SW_SHOW);
    Wnd := OldForm;
  end;
  EnableWindow(Wnd, SW_WHAT = SW_SHOW);
  {$IFDEF WIN32}ShowWindowAsync{$ELSE}ShowWindow{$ENDIF WIN32}(Wnd, SW_WHAT);
end;

procedure SwapFormsEx(Wnd1, Wnd2: HWND; SW_WHAT: Word);
var
  R: TRect;
begin
  GetWindowRect(Wnd1, R);
  MoveWindow(Wnd2, R.Left, R.Top, R.Right - R.Left, R.Bottom - R.Top, False);
  if SW_WHAT = SW_SHOW then
    ShowWindow(Wnd2, SW_SHOW);
end;

(*
function GetMemStuff: Longint;
const
  MemFileName: String[7] = 'MEM.TXT';
var
  FBatch: TextFile;
  P, M: PChar;
  Line: String;
  I: Integer;
begin
  try
    AssignFile(FBatch, 'SHELL.BAT');
    Rewrite(FBatch);
    writeln(FBatch, 'MEM >MEM.TXT');
    CloseFile(FBatch);
    if WaitAndClose_TTY(DoShellExecute(0, 'SHELL.BAT', nil, nil, SW_SHOWMINIMIZED),
      'Finished - Shell') then
    try
      P := ReadFileToBuffer(MemFileName);
      if not NilPtr(P) then
      begin
        M := StrPos(P, 'Total memory');
        if not NilPtr(M) then
        begin
          M[StrPos(M, #13#10)-M] := #0;
          Line := StrPas(M);
          I := 1;
          repeat
            if not (Line[I] in [' ', '0'..'9']) then
              System.Delete(Line, I, 1)
            else
              Inc(I);
          until (I > Length(Line));
          Line := TrimStr(Line);
          Result := StrToInt(Copy(Line, 1, pos(' ', Line)-1));
          {Delete(Line, 1, pos(' ', Line));Line:=TrimStr(Line);
          Mem2 := StrToInt(Copy(Line, 1, pos(' ', Line)-1));
          Delete(Line, 1, pos(' ', Line));Line:=TrimStr(Line);
          Mem3 := StrToInt(Line);}
        end;
      end;
    finally
      StrDispose(P);
    end;
  finally
    SysUtils.DeleteFile('SHELL.BAT');
    SysUtils.DeleteFile(MemFileName);
  end;
end;
*)
end.

The information provided above is provided "AS IS" without warranty of any kind.
Lucian Radulescu disclaims all warranties, either express or implied, including
the warranties of merchantability and fitness for a particular purpose.

In no event shall Lucian Radulescu or his suppliers be liable for any damages
whatsoever including direct, indirect, incidental, consequential, loss of business
profits or special damages, even if Lucian Radulescu or his suppliers have been
advised of the possibility of such damages.

Some states do not allow the exclusion or limitation of liability for consequential
or incidental damages so the foregoing limitation may not apply.

